from pytest import param

UA_DATA = [
    param(
        "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)",
        {"user_agents": ["Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)"]},
        {},
        id="user_agent_1",
    ),
    param(
        "mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11 mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0",
        {
            "user_agents": [
                "mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11",
                "mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0",
            ]
        },
        {},
        id="user_agent_2",
    ),
    param(
        "Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11 Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0",
        {
            "user_agents": [
                "Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11",
                "Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0",
            ]
        },
        {},
        id="user_agent_3",
    ),
]



"""Make sure that the parsing functions for specific functions are imported properly."""

from ioc_finder import parse_urls


def test_url_parsing_func():
    results = parse_urls("https://google.com")
    assert results == ["https://google.com"]



from pytest import param

URL_DATA = [
    param(
        "https://example.com/test%20page/foo.com/bingo.php?q=bar.com",
        {
            "urls": ["https://example.com/test%20page/foo.com/bingo.php?q=bar.com"],
            "domains": ["bar.com", "foo.com", "example.com"],
        },
        {},
        id="URL and domains parsed",
    ),
    param(
        "Foo https://citizenlab.ca/about/), bar",
        {
            "urls": ["https://citizenlab.ca/about/"],
        },
        {"parse_domain_from_url": False},
        id="URL boundary w/ ) handled properly",
    ),
    param(
        "DownloadString('https://example[.]com/rdp.ps1');g $I DownloadString(\"https://example[.]com/rdp.ps2\");g $I",
        {
            "urls": ["https://example.com/rdp.ps1", "https://example.com/rdp.ps2"],
        },
        {"parse_domain_from_url": False},
        id="URL boundary w/ single or double quotes handled properly",
    ),
    param(
        "https://example.com/g//foo",
        {
            "urls": ["https://example.com/g//foo"],
        },
        {"parse_domain_from_url": False},
        id="Consecutive slashes handled properly",
    ),
]



#!/usr/bin/env python3

from .ioc_finder import (
    find_iocs,
    parse_asns,
    parse_authentihashes_,
    parse_bitcoin_addresses,
    parse_complete_email_addresses,
    parse_cves,
    parse_domain_names,
    parse_email_addresses,
    parse_enterprise_attack_tactics,
    parse_enterprise_attack_techniques,
    parse_file_paths,
    parse_google_adsense_ids,
    parse_google_analytics_ids,
    parse_imphashes_,
    parse_ipv4_addresses,
    parse_ipv4_cidrs,
    parse_ipv6_addresses,
    parse_mac_addresses,
    parse_md5s,
    parse_mobile_attack_tactics,
    parse_mobile_attack_techniques,
    parse_pre_attack_tactics,
    parse_pre_attack_techniques,
    parse_registry_key_paths,
    parse_sha1s,
    parse_sha256s,
    parse_sha512s,
    parse_ssdeeps,
    parse_tlp_labels,
    parse_urls,
    parse_user_agents,
    parse_xmpp_addresses,
    prepare_text,
)

__all__ = [
    "find_iocs",
    "parse_asns",
    "parse_authentihashes_",
    "parse_bitcoin_addresses",
    "parse_complete_email_addresses",
    "parse_cves",
    "parse_domain_names",
    "parse_email_addresses",
    "parse_enterprise_attack_tactics",
    "parse_enterprise_attack_techniques",
    "parse_file_paths",
    "parse_google_adsense_ids",
    "parse_google_analytics_ids",
    "parse_imphashes_",
    "parse_ipv4_addresses",
    "parse_ipv4_cidrs",
    "parse_ipv6_addresses",
    "parse_mac_addresses",
    "parse_md5s",
    "parse_mobile_attack_tactics",
    "parse_mobile_attack_techniques",
    "parse_pre_attack_tactics",
    "parse_pre_attack_techniques",
    "parse_registry_key_paths",
    "parse_sha1s",
    "parse_sha256s",
    "parse_sha512s",
    "parse_ssdeeps",
    "parse_tlp_labels",
    "parse_urls",
    "parse_user_agents",
    "parse_xmpp_addresses",
    "prepare_text",
]

__author__ = """Floyd Hightower"""
__version__ = "7.3.0"



from ioc_finder import find_iocs


def test_tlp_labels():
    s = "tlp amber and TLP:RED"
    iocs = find_iocs(s)
    assert len(iocs["tlp_labels"]) == 2
    assert "TLP:RED" in iocs["tlp_labels"]
    assert "TLP:AMBER" in iocs["tlp_labels"]

    s = "tlp-Amber and TLPRED TlpGreen"
    iocs = find_iocs(s)
    assert len(iocs["tlp_labels"]) == 3
    assert "TLP:RED" in iocs["tlp_labels"]
    assert "TLP:AMBER" in iocs["tlp_labels"]
    assert "TLP:GREEN" in iocs["tlp_labels"]


def test_domain_parsing():
    s = "this is just a (google.com) test of example.com"
    iocs = find_iocs(s)
    assert len(iocs["domains"]) == 2
    assert "google.com" in iocs["domains"]


def test_ipv4_parsing():
    s = "this is just a (1.2.3.54) test of 255.255.1.255 255.256.344.1"
    iocs = find_iocs(s)
    assert len(iocs["ipv4s"]) == 2
    assert "1.2.3.54" in iocs["ipv4s"]
    assert "255.255.1.255" in iocs["ipv4s"]
    assert "255.256.344.1" not in iocs["ipv4s"]


def test_ipv6_parsing():
    s = "2001:0db8:0000:0000:0000:ff00:0042:8329 testing 2001:db8:0:0:0:ff00:42:8329 shfaldkafsdfa 2001:db8::ff00:42:8329 asdfadfas afkj;fl ::1 kljfkadf 1:1"
    iocs = find_iocs(s)
    assert len(iocs["ipv6s"]) == 4
    # TODO: the following 3 ipv6s addresses are the same representation of the same thing; I need to deduplicate these more thoroughly in the parsing function
    assert "2001:0db8:0000:0000:0000:ff00:0042:8329" in iocs["ipv6s"]
    assert "2001:db8:0:0:0:ff00:42:8329" in iocs["ipv6s"]
    assert "2001:db8::ff00:42:8329" in iocs["ipv6s"]
    assert "::1" in iocs["ipv6s"]
    assert "1:1" not in iocs["ipv6s"]


def test_email_address_parsing():
    s = "test@a.com bingo@en.wikipedia.com foo@a.com'.format('a'*63 bar@b.a.com'.format('a'*63, 'a'*63 bad@test-ing.com me@2600.com john.smith(comment)@example.com (comment)john.smith@example.com \"John..Doe\"@example.com' test@[192.168.0.1]"

    iocs = find_iocs(s)["email_addresses_complete"]
    print(iocs)
    assert len(iocs) == 10
    assert "test@a.com" in iocs
    assert "bingo@en.wikipedia.com" in iocs
    assert "foo@a.com" in iocs
    assert "bar@b.a.com" in iocs
    assert "bad@test-ing.com" in iocs
    assert "me@2600.com" in iocs
    assert "john.smith(comment)@example.com" in iocs
    assert "(comment)john.smith@example.com" in iocs
    assert '"John..Doe"@example.com' in iocs
    assert "test@[192.168.0.1]" in iocs

    iocs = find_iocs("a@example.com")
    assert iocs["email_addresses_complete"][0] == "a@example.com"


def test_complex_email_address_parsing():
    s = "john.smith(comment)@example.com"
    iocs = find_iocs(s)
    assert "john.smith(comment)@example.com" in iocs["email_addresses_complete"]


def test_simple_email_address_parsing():
    s = "test@a.com bingo@en.wikipedia.com foo@a.com'.format('a'*63 bar@b.a.com'.format('a'*63, 'a'*63 bad@test-ing.com me@2600.com john.smith(comment)@example.com (comment)john.smith@example.com \"John..Doe\"@example.com' test@[192.168.0.1]"

    iocs = find_iocs(s)
    assert len(iocs["email_addresses"]) == 8
    assert "test@a.com" in iocs["email_addresses"]
    assert "bingo@en.wikipedia.com" in iocs["email_addresses"]
    assert "foo@a.com" in iocs["email_addresses"]
    assert "bar@b.a.com" in iocs["email_addresses"]
    assert "bad@test-ing.com" in iocs["email_addresses"]
    assert "me@2600.com" in iocs["email_addresses"]
    assert "john.smith@example.com" in iocs["email_addresses"]
    assert "test@[192.168.0.1]" in iocs["email_addresses"]

    iocs = find_iocs("a@example.com")
    assert iocs["email_addresses"][0] == "a@example.com"


def test_url_parsing():
    invalid_urls = [
        "foo@{}.com".format("a" * 64),
        "!@.com",
        "foo@abc",
        "@@@@-hi-.com",
        "@_hi_.com",
        "me@*hi*.com",
        "foo{}.com".format("a" * 64),
        ".com",
        "abc",
        "-hi-.com",
        "_hi_.com",
        "*hi*.com",
    ]

    valid_urls = [
        "https://a.com",
        "https://en.wikipedia.com",
        "https://{}.com".format("a" * 63),
        "https://{}.{}.com".format("a" * 63, "a" * 63),
        "https://test-ing.com",
        "https://2600.com",
        "https://example.com",
        "http://example.com",
        "ftp://example.com",
    ]

    iocs = find_iocs(" ".join(valid_urls))
    assert len(iocs["urls"]) == 9
    # make sure domains are being parsed from the valid urls as well
    assert len(iocs["domains"]) == 7

    iocs = find_iocs(" ".join(invalid_urls))
    assert len(iocs["urls"]) == 0

    s = "http://8pretgdl.r.us-east-1.awstrack.me/L0/http:%2F%2Fwww.excelgoodies.com%2Fexcel-vba-training-in-virginia%23course-content/1/0100016ed23f4bef-b14931bd-26f6-4130-9c37-c4f9902a771d-000000/mHJBuJ8D1RcIDE3jrWkdw4I9im4=138"
    iocs = find_iocs(s)
    print(iocs["urls"])
    assert iocs["urls"] == [
        "http://8pretgdl.r.us-east-1.awstrack.me/L0/http:%2F%2Fwww.excelgoodies.com%2Fexcel-vba-training-in-virginia%23course-content/1/0100016ed23f4bef-b14931bd-26f6-4130-9c37-c4f9902a771d-000000/mHJBuJ8D1RcIDE3jrWkdw4I9im4=138"
    ]

    s = "https://www.virustotal.com/gui/file/2f3ec0e4998909bb0efab13c82d30708ca9f88679e42b75ef13ea0466951d862/detection"
    iocs = find_iocs(s)
    assert iocs["sha256s"] == ["2f3ec0e4998909bb0efab13c82d30708ca9f88679e42b75ef13ea0466951d862"]

    # this was implemented for https://github.com/fhightower/ioc-finder/issues/87
    s = "https://www.virustotal.com/gui/file/2f3ec0e4998909bb0efab13c82d30708ca9f88679e42b75ef13ea0466951d862/detection"
    iocs = find_iocs(s, parse_from_url_path=False)
    assert iocs["sha256s"] == []

    # this was implemented for https://github.com/fhightower/ioc-finder/issues/87
    s = "https://www.virustotal.com/gui/file/2f3ec0e4998909bb0efab13c82d30708ca9f88679e42b75ef13ea0466951d862/detection"
    iocs = find_iocs(s, parse_from_url_path=False, parse_urls_without_scheme=False)
    assert iocs["urls"] == [
        "https://www.virustotal.com/gui/file/2f3ec0e4998909bb0efab13c82d30708ca9f88679e42b75ef13ea0466951d862/detection"
    ]
    assert iocs["sha256s"] == []


def test_file_hash_parsing():
    s = "{} {} {} {} {}".format("A" * 32, "a" * 32, "b" * 40, "c" * 64, "d" * 128)
    iocs = find_iocs(s)
    assert len(iocs["md5s"]) == 1
    assert len(iocs["sha1s"]) == 1
    assert len(iocs["sha256s"]) == 1
    assert len(iocs["sha512s"]) == 1


def test_cve_parsing():
    s = "cve-2014-1000 cve 2014-1001 cve-1999-1002 CVE 2999-1003 CVE 1928-1004"
    iocs = find_iocs(s)
    assert len(iocs["cves"]) == 5
    assert "CVE-2014-1000" in iocs["cves"]
    assert "CVE-2014-1001" in iocs["cves"]
    assert "CVE-1999-1002" in iocs["cves"]
    assert "CVE-2999-1003" in iocs["cves"]
    assert "CVE-1928-1004" in iocs["cves"]


def test_ipv4_cidr_parsing():
    s = "1.2.3.4/0 1.2.3.4/10 1.2.3.4/20 1.2.3.4/32"
    iocs = find_iocs(s)
    assert len(iocs["ipv4_cidrs"]) == 4
    assert "1.2.3.4/0" in iocs["ipv4_cidrs"]
    assert "1.2.3.4/10" in iocs["ipv4_cidrs"]
    assert "1.2.3.4/20" in iocs["ipv4_cidrs"]
    assert "1.2.3.4/32" in iocs["ipv4_cidrs"]

    s = "1.2.3.4/0 1.2.3.4/10 1.2.3.4/20 1.2.3.4/32"
    iocs = find_iocs(s, parse_address_from_cidr=False)
    assert len(iocs["ipv4_cidrs"]) == 4
    assert len(iocs["ipv4s"]) == 0


def test_registry_key_parsing():
    s = r"HKEY_LOCAL_MACHINE\Software\Microsoft\Windows HKLM\Software\Microsoft\Windows HKCC\Software\Microsoft\Windows"
    iocs = find_iocs(s)
    assert sorted(
        [
            r"HKEY_LOCAL_MACHINE\Software\Microsoft\Windows",
            r"HKLM\Software\Microsoft\Windows",
            r"HKCC\Software\Microsoft\Windows",
        ]
    ) == sorted(iocs["registry_key_paths"])


def test_adsense_publisher_id_parsing():
    s = "pub-1234567891234567"
    iocs = find_iocs(s)
    assert len(iocs["google_adsense_publisher_ids"]) == 1
    iocs["google_adsense_publisher_ids"][0] == "pub-1234567891234567"

    s = "pub-1234567891234567 pub-9383614236930773"
    iocs = find_iocs(s)
    assert len(iocs["google_adsense_publisher_ids"]) == 2
    assert "pub-1234567891234567" in iocs["google_adsense_publisher_ids"]
    assert "pub-9383614236930773" in iocs["google_adsense_publisher_ids"]


def test_analytics_publisher_id_parsing():
    s = "UA-000000-2"
    iocs = find_iocs(s)
    assert len(iocs["google_analytics_tracker_ids"]) == 1
    assert iocs["google_analytics_tracker_ids"][0] == "UA-000000-2"

    s = "UA-000000-2 UA-00000000-99"
    iocs = find_iocs(s)
    assert len(iocs["google_analytics_tracker_ids"]) == 2
    assert "UA-000000-2" in iocs["google_analytics_tracker_ids"]
    assert "UA-00000000-99" in iocs["google_analytics_tracker_ids"]


def test_bitcoin_parsing():
    s = """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq"""
    iocs = find_iocs(s)
    assert len(iocs["bitcoin_addresses"]) == 3
    assert "1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2" in iocs["bitcoin_addresses"]
    assert "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy" in iocs["bitcoin_addresses"]
    assert "bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq" in iocs["bitcoin_addresses"]


def test_xmpp_address_parsing():
    s = """foo@swissjabber.de bar@jabber.zone bom@jabber.sow.as me@example.com"""
    iocs = find_iocs(s)
    assert len(iocs["xmpp_addresses"]) == 3
    assert "foo@swissjabber.de" in iocs["xmpp_addresses"]
    assert "bar@jabber.zone" in iocs["xmpp_addresses"]
    assert "bom@jabber.sow.as" in iocs["xmpp_addresses"]
    assert len(iocs["domains"]) == 4
    # make sure the xmpp addresses are not also parsed as email addresses
    assert len(iocs["email_addresses"]) == 1

    iocs = find_iocs(s, parse_domain_name_from_xmpp_address=False)
    assert len(iocs["xmpp_addresses"]) == 3
    assert "foo@swissjabber.de" in iocs["xmpp_addresses"]
    assert "bar@jabber.zone" in iocs["xmpp_addresses"]
    assert "bom@jabber.sow.as" in iocs["xmpp_addresses"]
    assert len(iocs["domains"]) == 1
    # make sure the xmpp addresses are not also parsed as email addresses
    assert len(iocs["email_addresses"]) == 1


def test_mac_address_parsing():
    s = "AA-F2-C9-A6-B3-4F AB:F2:C9:A6:B3:4F ACF2.C9A6.B34F"

    iocs = find_iocs(s)
    assert len(iocs["mac_addresses"]) == 3
    assert "AA-F2-C9-A6-B3-4F" in iocs["mac_addresses"]
    assert "AB:F2:C9:A6:B3:4F" in iocs["mac_addresses"]
    assert "ACF2.C9A6.B34F" in iocs["mac_addresses"]

    # same thing, just lower-case
    s = "aa-f2-c9-a6-b3-4f ab:f2:c9:a6:b3:4f acf2.c9a6.b34f"
    iocs = find_iocs(s)
    assert len(iocs["mac_addresses"]) == 3
    assert "aa-f2-c9-a6-b3-4f" in iocs["mac_addresses"]
    assert "ab:f2:c9:a6:b3:4f" in iocs["mac_addresses"]
    assert "acf2.c9a6.b34f" in iocs["mac_addresses"]


def test_ssdeep_parsing():
    s = "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"
    iocs = find_iocs(s)
    assert len(iocs["ssdeeps"]) == 1
    assert iocs["ssdeeps"][0] == "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"

    s = "ahdfadsfa 1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,000/000/000000001 adfasf"
    iocs = find_iocs(s)
    assert len(iocs["ssdeeps"]) == 1
    assert iocs["ssdeeps"][0] == "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"

    s = """c2b257868686c861d43c6cf3de146b8812778c8283f7d
Threat  Zepakab/Zebrocy Downloader
ssdeep  12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F"""
    iocs = find_iocs(s)
    assert len(iocs["ssdeeps"]) == 1
    assert iocs["ssdeeps"][0] == "12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F"

    s = """393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b,"000/000/000000001"
196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg,"000/000/000000002"
98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S,"000/000/000000003"
96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg,"000/000/000000004"
196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax,"000/000/000000005"
1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6,"000/000/000000006"
48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ,"000/000/000000007"
24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ,"000/000/000000008"
1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,"000/000/000000009"""
    iocs = find_iocs(s)
    assert len(iocs["ssdeeps"]) == 9
    assert (
        "393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b"
        in iocs["ssdeeps"]
    )
    assert "196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg" in iocs["ssdeeps"]
    assert (
        "98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S"
        in iocs["ssdeeps"]
    )
    assert "96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg" in iocs["ssdeeps"]
    assert (
        "196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax" in iocs["ssdeeps"]
    )
    assert (
        "1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6"
        in iocs["ssdeeps"]
    )
    assert (
        "48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ" in iocs["ssdeeps"]
    )
    assert (
        "24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ"
        in iocs["ssdeeps"]
    )
    assert "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U" in iocs["ssdeeps"]


def test_imphash_parsing():
    names = ["imphash", "import hash"]
    templates = [
        """SHA-256 093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a
Dropper {}: 18ddf28a71089acdbab5038f58044c0a
C2 IP: 210.209.127.8:443""",
        "{}: 18ddf28a71089acdbab5038f58044c0a",
        "{} 18ddf28a71089acdbab5038f58044c0a",
        "{}  18ddf28a71089acdbab5038f58044c0a",
        "{}:     18ddf28a71089acdbab5038f58044c0a",
        "{}\t18ddf28a71089acdbab5038f58044c0a",
        "{}\n18ddf28a71089acdbab5038f58044c0a",
        "{} - 18ddf28a71089acdbab5038f58044c0a",
    ]

    for template in templates:
        for name in names:
            print(template)
            iocs = find_iocs(template.format(name))
            assert len(iocs["imphashes"]) == 1
            assert iocs["imphashes"] == ["18ddf28a71089acdbab5038f58044c0a"]
            assert len(iocs["md5s"]) == 0

            iocs = find_iocs(template.format(name.upper()))
            assert len(iocs["imphashes"]) == 1
            assert iocs["imphashes"] == ["18ddf28a71089acdbab5038f58044c0a"]
            assert len(iocs["md5s"]) == 0


def test_authentihash():
    names = ["authentihash"]
    templates = [
        "{} 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}   3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}: 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}:     3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{} - 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}-3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}\t3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
        "{}\n3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
    ]

    for template in templates:
        for name in names:
            print(template)
            iocs = find_iocs(template.format(name))
            assert len(iocs["authentihashes"]) == 1
            assert iocs["authentihashes"] == ["3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4"]
            assert len(iocs["sha256s"]) == 0

            iocs = find_iocs(template.format(name.upper()))
            assert len(iocs["authentihashes"]) == 1
            assert iocs["authentihashes"] == ["3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4"]
            assert len(iocs["sha256s"]) == 0


def test_user_agents():
    s = "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)"
    iocs = find_iocs(s)
    assert len(iocs["user_agents"]) == 1
    assert iocs["user_agents"] == ["Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)"]

    s = "mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11 mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0"
    iocs = find_iocs(s)
    assert len(iocs["user_agents"]) == 2
    assert (
        "mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11"
        in iocs["user_agents"]
    )
    assert "mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0" in iocs["user_agents"]

    # test the same thing as above but with different casing to make sure the cases are matched and maintained properly
    s = "Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11 Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0"
    iocs = find_iocs(s)
    assert len(iocs["user_agents"]) == 2
    assert (
        "Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11"
        in iocs["user_agents"]
    )
    assert "Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0" in iocs["user_agents"]


def test_monero_addresses():
    result = find_iocs(
        "496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp"
    )
    assert result["monero_addresses"] == [
        "496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp"
    ]

    s = "49Bmp3SfddJRRGNW7GhHyAA2JgcYmZ4EGEix6p3eMNFCd15P2VsK9BHWcZWUNYF3nhf17MoRTRK4j5b7FUMA9zanSn9D3Nk 498s2XeKWYSEhQHGxdMULWdrpaKvSkDsq4855mCuksNL6ez2dk4mMQm8epbr9xvn5LgLPzD5uL9EGeRqWUdEZha1HmZqcyh"
    result = find_iocs(s)
    assert (
        "49Bmp3SfddJRRGNW7GhHyAA2JgcYmZ4EGEix6p3eMNFCd15P2VsK9BHWcZWUNYF3nhf17MoRTRK4j5b7FUMA9zanSn9D3Nk"
        in result["monero_addresses"]
    )
    assert (
        "498s2XeKWYSEhQHGxdMULWdrpaKvSkDsq4855mCuksNL6ez2dk4mMQm8epbr9xvn5LgLPzD5uL9EGeRqWUdEZha1HmZqcyh"
        in result["monero_addresses"]
    )



#!/usr/bin/env python3

import pytest

from ioc_finder import find_iocs


@pytest.fixture
def text_a():
    """Provide some generic text for the tests below."""
    return "example.com is a nice domain if you consider http://bad.com/test/bingo.php to be bad. {} {} {} 1.2.3.4 192.64.55.61 bad12312@example.org".format(
        "a" * 32, "b" * 40, "c" * 64
    )


def test_domain_name_with_underscore():
    # see https://github.com/fhightower/ioc-finder/issues/26
    s = "o_o.lgms.nl"
    results = find_iocs(s)
    assert results["domains"] == ["o_o.lgms.nl"]

    s = "_jabber._tcp.gmail.com"
    results = find_iocs(s)
    assert results["domains"] == ["_jabber._tcp.gmail.com"]


def test_url_with_underscore_in_subdomain():
    # see https://github.com/fhightower/ioc-finder/issues/26
    s = "https://o_o.lgms.nl/"
    results = find_iocs(s)
    assert results["urls"] == ["https://o_o.lgms.nl/"]


def test_ioc_finder(text_a):
    iocs = find_iocs(text_a)
    assert len(iocs["domains"]) == 3
    assert "example.com" in iocs["domains"]
    assert "example.org" in iocs["domains"]
    assert "bad.com" in iocs["domains"]

    assert iocs["email_addresses_complete"] == ["bad12312@example.org"]

    assert len(iocs["ipv4s"]) == 2
    assert "1.2.3.4" in iocs["ipv4s"]
    assert "192.64.55.61" in iocs["ipv4s"]

    assert iocs["urls"] == ["http://bad.com/test/bingo.php"]
    assert iocs["md5s"] == ["aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"]
    assert iocs["sha1s"] == ["bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb"]
    assert iocs["sha256s"] == ["cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc"]


def test_url_parsing():
    """Test some specific url examples."""
    s = "https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"]

    s = '<link href="http://fonts.googleapis.com/css?family=Lato:400,700" rel="stylesheet" type="text/css"/>'
    iocs = find_iocs(s)
    assert "http://fonts.googleapis.com/css?family=Lato:400,700" in iocs["urls"]

    s = '<link href="http://fonts.googleapis.com/css?family=Lato:400,700" rel="stylesheet" type="text/css"/>'
    iocs = find_iocs(s)
    assert "http://fonts.googleapis.com/css?family=Lato:400,700" in iocs["urls"]

    s = '<a href="https://bit.ly/12345#abcd" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results["urls"] == ["https://bit.ly/12345#abcd"]

    s = '<a href="https://bit.ly/12345" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results["urls"] == ["https://bit.ly/12345"]

    s = '<a href="https://bit.ly" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results["urls"] == ["https://bit.ly"]

    s = '<a href="https://bit.ly/" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results["urls"] == ["https://bit.ly/"]

    s = "http://example.com//test"
    results = find_iocs(s)
    assert results["urls"] == ["http://example.com//test"]


def test_issue_45_url_parsing():
    s = "http://wmfolcs3.pn.4y.nv.kr2x1dt.net/gz+/(y%40%26//%3c7aew%5cqv%0a/%0bcz,r/r%5c%7b/7re//6%3e/f%23%7ce0p'6_%09/d%5c"
    results = find_iocs(s)
    assert results["urls"] == [
        "http://wmfolcs3.pn.4y.nv.kr2x1dt.net/gz+/(y%40%26//%3c7aew%5cqv%0a/%0bcz,r/r%5c%7b/7re//6%3e/f%23%7ce0p'6_%09/d%5c"
    ]


def test_schemeless_url_parsing():
    """Test parsing URLs without a scheme."""
    s = "github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"]

    s = "github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh hightower.space/projects"
    iocs = find_iocs(s)
    assert len(iocs["urls"]) == 2
    assert "hightower.space/projects" in iocs["urls"]
    assert "github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh" in iocs["urls"]

    s = "https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh hightower.space/projects"
    iocs = find_iocs(s, parse_urls_without_scheme=False)
    assert iocs["urls"] == ["https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"]


def test_address_email_address():
    s = ">test@[192.168.2.1]<"
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ["test@[192.168.2.1]"]
    assert iocs["email_addresses"] == ["test@[192.168.2.1]"]
    assert iocs["ipv4s"] == ["192.168.2.1"]

    s = "bad@[192.168.7.3]"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["192.168.7.3"]
    assert iocs["email_addresses_complete"] == ["bad@[192.168.7.3]"]
    assert iocs["email_addresses"] == ["bad@[192.168.7.3]"]

    s = "bad@[192.168.7.3]aaaaa"
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ["bad@[192.168.7.3]"]
    assert iocs["email_addresses"] == ["bad@[192.168.7.3]"]

    s = "jsmith@[IPv6:2001:db8::1]"
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ["jsmith@[IPv6:2001:db8::1]"]
    assert iocs["email_addresses"] == ["jsmith@[IPv6:2001:db8::1]"]
    assert iocs["ipv6s"] == ["2001:db8::1"]


def test_address_domain_url():
    s = "http://192.64.55.61/test.php"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["http://192.64.55.61/test.php"]
    assert iocs["ipv4s"] == ["192.64.55.61"]


def test_url_domain_name_parsing():
    s = "http://foo.youtube/test.php"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["http://foo.youtube/test.php"]
    assert iocs["domains"] == ["foo.youtube"]


def test_ioc_deduplication():
    """Make sure the results returned from the ioc_finder are deduplicated."""
    iocs = find_iocs("example.com example.com")
    assert iocs["domains"] == ["example.com"]


def test_file_hash_order():
    s = "{} {}".format("a" * 32, "b" * 40)
    iocs = find_iocs(s)
    assert iocs["md5s"][0] == "a" * 32
    assert iocs["sha1s"][0] == "b" * 40


def test_file_hash_parsing():
    s = "this is a test{}".format("a" * 32)
    iocs = find_iocs(s)
    assert iocs["md5s"] == []

    s = "this is a test {}".format("a" * 32)
    iocs = find_iocs(s)
    assert iocs["md5s"] == ["a" * 32]

    s = 'this is a test "{}"'.format("a" * 32)
    iocs = find_iocs(s)
    assert iocs["md5s"] == ["a" * 32]

    s = "this is a test {}.".format("a" * 32)
    iocs = find_iocs(s)
    assert iocs["md5s"] == ["a" * 32]

    s = "0x1a1db93766e31994507511c9c70a1dd94465cf6d"
    iocs = find_iocs(s)
    assert iocs["sha1s"] == ["1a1db93766e31994507511c9c70a1dd94465cf6d"]


def test_url_boundaries():
    """Make sure the boundaries for a url are correct."""
    s = """http://192.168.0.1/test/bad.html</a><br></div>"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["http://192.168.0.1/test/bad.html"]

    s = """<IMg SRc="https://i.imgur.com/abc.png#4827766048"/><br>
<IMg SRc="https://i.imgur.com/def.png#4827766048"/><br>"""
    iocs = find_iocs(s)
    assert "https://i.imgur.com/abc.png#4827766048" in iocs["urls"]
    assert "https://i.imgur.com/def.png#4827766048" in iocs["urls"]
    assert len(iocs["urls"]) == 2

    s = """<IMg SRc="https://i.imgur.com/abc.png"/><br>"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://i.imgur.com/abc.png"]

    s = """(https://i.imgur.com/abc.png)"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://i.imgur.com/abc.png"]

    s = """(https://i.imgur.com/abc.png#abc)"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://i.imgur.com/abc.png#abc"]

    s = """[https://i.imgur.com/abc.png](https://i.imgur.com/abc.png)"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://i.imgur.com/abc.png"]

    s = """[https://i.imgur.com/abc.png#abc](https://i.imgur.com/abc.png#abc)"""
    iocs = find_iocs(s)
    assert iocs["urls"] == ["https://i.imgur.com/abc.png#abc"]


def test_domain_parsing():
    s = "Host: dfasdfa (mz-fcb301p.ocn.ad.jp asdfsdafs"
    iocs = find_iocs(s)
    assert iocs["domains"] == ["mz-fcb301p.ocn.ad.jp"]

    s = "smtp.mailfrom"
    iocs = find_iocs(s)
    assert iocs["domains"] == []

    s = "bar.com"
    iocs = find_iocs(s)
    assert iocs["domains"] == ["bar.com"]

    s = 'bar.com"'
    iocs = find_iocs(s)
    assert iocs["domains"] == ["bar.com"]

    s = "bar.com'"
    iocs = find_iocs(s)
    assert iocs["domains"] == ["bar.com"]

    s = "bar.com."
    iocs = find_iocs(s)
    assert iocs["domains"] == ["bar.com"]

    # make sure domains of different casings are properly parsed: https://github.com/fhightower/ioc-finder/issues/47
    iocs = find_iocs("BAR.com")
    assert iocs["domains"] == ["bar.com"]
    iocs = find_iocs("bar.COM")
    assert iocs["domains"] == ["bar.com"]
    iocs = find_iocs("BAR.COM")
    assert iocs["domains"] == ["bar.com"]


def test_email_address_parsing():
    s = 'my email is: foo"bar@gmail.com'
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ['foo"bar@gmail.com']
    assert iocs["email_addresses"] == ["bar@gmail.com"]

    s = "Abc\\@def@example.com"
    iocs = find_iocs(s)
    print(iocs["email_addresses_complete"])
    print(iocs["email_addresses"])
    assert iocs["email_addresses_complete"] == ["Abc\\@def@example.com"]
    assert iocs["email_addresses"] == ["def@example.com"]

    s = 'foobar@gmail.com"'
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ["foobar@gmail.com"]
    assert iocs["email_addresses"] == ["foobar@gmail.com"]

    s = "foobar@gmail.comahhhhhhhh"
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == []
    assert iocs["email_addresses"] == []

    s = '"foobar@gmail.com'
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ['"foobar@gmail.com']
    assert iocs["email_addresses"] == ["foobar@gmail.com"]

    s = "smtp.mailfrom=example@example.com"
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ["smtp.mailfrom=example@example.com"]
    assert iocs["email_addresses"] == ["example@example.com"]

    s = '"foo@bar.com"'
    iocs = find_iocs(s)
    assert iocs["email_addresses_complete"] == ['"foo@bar.com']
    assert iocs["email_addresses"] == ["foo@bar.com"]

    # making sure that the `parse_domain_from_email_address` argument is working properly
    s = "foo@bar.com."
    iocs = find_iocs(s, parse_domain_from_email_address=False)
    assert iocs["email_addresses"] == ["foo@bar.com"]
    assert iocs["domains"] == []

    s = '"foo@bar.com'
    iocs = find_iocs(s)
    assert iocs["email_addresses"] == ["foo@bar.com"]

    # validating https://github.com/fhightower/ioc-finder/issues/40 is fixed
    s = "-----foo@bar.com"
    iocs = find_iocs(s)
    assert iocs["email_addresses"] == ["foo@bar.com"]

    s = "foo-burt@bar.com f-1@bar.com"
    iocs = find_iocs(s)
    assert len(iocs["email_addresses"]) == 2
    assert "foo-burt@bar.com" in iocs["email_addresses"]
    assert "f-1@bar.com" in iocs["email_addresses"]


def test_erroneous_ip_address_parsing():
    # the two tests below make sure that IP addresses are not parsed from strings with decimals in them
    s = "2018.12.15.14.05.43"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "111.12.15.14.05.43"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = ".18.12.15.14"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "18.12.15.1411111111"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "018.12.15.14"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["18.12.15.14"]

    s = "18.12.15.14."
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["18.12.15.14"]

    # the three tests below make sure that IP addresses are not parsed from sequences with large numbers in them
    s = "1112.15.14.05"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "15.1112.14.05"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "15.14.05.1112"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []


def test_ip_address_systematically():
    # TODO: for many of the assertions below, I would like to be more explicit; I would like to change tests like `len(iocs['ipv4s']) == 1` to `iocs['ipv4s'] == ['1.1.1.1']`
    s = "1.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["1.1.1.1"]

    s = ".1.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    # I would like to match in this situation to capture ip address that are at the end of a sentence
    s = "1.1.1.1."
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["1.1.1.1"]

    s = ".1.1.1.1."
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "1.1.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = ".1.1.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "1.1.1.1.1."
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = ".1.1.1.1.1."
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "1.1.1.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "1.1.1.1.a"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []

    s = "1.01.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["1.1.1.1"]

    s = "01.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["1.1.1.1"]

    s = "01.01.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == ["1.1.1.1"]

    s = "0001.1.1.1"
    iocs = find_iocs(s)
    assert iocs["ipv4s"] == []


def test_onion_parsing():
    s = "foo.onion"
    iocs = find_iocs(s)
    assert iocs["domains"] == ["foo.onion"]

    s = "http://foo.onion/test"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["http://foo.onion/test"]
    assert iocs["domains"] == ["foo.onion"]


def test_deduplication_of_indicators_with_different_cases():
    s = "example.com Example.com exAmplE.com"
    iocs = find_iocs(s)
    assert iocs["domains"] == ["example.com"]

    s = "bad@example.com bad@Example.com bad@exAmplE.com"
    iocs = find_iocs(s)
    assert iocs["email_addresses"] == ["bad@example.com"]

    s = "bad@example.com Bad@example.com"
    iocs = find_iocs(s)
    assert iocs["email_addresses"] == ["bad@example.com"]

    s = "http://example.com/test http://EXAMple.com/test"
    iocs = find_iocs(s)
    assert iocs["urls"] == ["http://example.com/test"]

    s = "http://example.com/test http://EXAMple.com/TEST"
    iocs = find_iocs(s)
    assert len(iocs["urls"]) == 2
    assert "http://example.com/test" in iocs["urls"]
    assert "http://example.com/TEST" in iocs["urls"]


def test_google_adsense_publisher_ids():
    s = "PUB-1234567891234567"
    iocs = find_iocs(s)
    assert iocs["google_adsense_publisher_ids"] == ["pub-1234567891234567"]

    s = "pUb-1234567891234567"
    iocs = find_iocs(s)
    assert iocs["google_adsense_publisher_ids"] == []

    s = "PUB-1234567891234567"
    iocs = find_iocs(s)
    assert iocs["google_adsense_publisher_ids"] == ["pub-1234567891234567"]


def test_google_analyitics_tracker_ids():
    s = "ua-000000-1"
    iocs = find_iocs(s)
    assert iocs["google_analytics_tracker_ids"] == ["UA-000000-1"]

    s = "uA-000000-1"
    iocs = find_iocs(s)
    assert iocs["google_analytics_tracker_ids"] == []

    s = "UA-000000-1"
    iocs = find_iocs(s)
    assert iocs["google_analytics_tracker_ids"] == ["UA-000000-1"]


def test_google_casing_deduplication():
    s = "pub-1234567891234567 PUB-1234567891234567 pUb-1234567891234567"
    iocs = find_iocs(s)
    assert iocs["google_adsense_publisher_ids"] == ["pub-1234567891234567"]

    s = "UA-000000-1 ua-000000-1"
    iocs = find_iocs(s)
    assert iocs["google_analytics_tracker_ids"] == ["UA-000000-1"]


def test_not_parsing_imphash():
    s = "imphash 18ddf28a71089acdbab5038f58044c0a"
    iocs = find_iocs(s, parse_imphashes=False)
    assert "imphashes" not in iocs
    # even if we aren't parsing imphashes, they will still be removed and, thus, not parsed as md5s
    assert iocs["md5s"] == []


def test_not_parsing_authentihash():
    s = "authentihash 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4"
    iocs = find_iocs(s, parse_authentihashes=False)
    assert "authentihashes" not in iocs
    # even if we aren't parsing authentihashes, they will still be removed and, thus, not parsed as sha256s
    assert iocs["sha256s"] == []


def test_mac_address_parsing():
    s = "2019.02.15"
    iocs = find_iocs(s)
    assert iocs["mac_addresses"] == []


def test_unix_file_paths__not_detect_url():
    # https://github.com/fhightower/ioc-finder/issues/42
    s = "https://twitter.com/"
    iocs = find_iocs(s)
    assert iocs["file_paths"] == []


def test_ipv6_parsing():
    # https://github.com/fhightower/ioc-finder/issues/37
    s = "11:04:10 -0500"
    iocs = find_iocs(s)
    assert iocs["ipv6s"] == []


def test_ssdeep_parsing():
    # https://github.com/fhightower/ioc-finder/issues/36
    s = "11:04:10 -0500"
    iocs = find_iocs(s)
    assert iocs["ssdeeps"] == []


def test_certificate_serial_number_issue_96():
    # see https://github.com/fhightower/ioc-finder/issues/96
    s = """SolarWinds.Orion.Core.BusinessLayer.dll is signed by SolarWinds, using the certificate with serial number 0f:e9:73:75:20:22:a6:06:ad:f2:a3:6e:34:5d:c0:ed. The file was signed on March 24, 2020."""
    observables = find_iocs(s)
    print(observables)
    assert observables["ipv6s"] == []
    assert observables["mac_addresses"] == []



"""Using hypothesis (https://hypothesis.readthedocs.io/en/latest/index.html) to test the finder."""

from hypothesis import given, settings
from hypothesis.provisional import domains, urls
from hypothesis.strategies._internal.ipaddress import ip_addresses

from ioc_finder import find_iocs

# @given(urls())
# @settings(deadline=None)
# def test_url_parsing(url):
#     url = url.lower()
#     iocs = find_iocs(url)
#     failure = False

#     try:
#         assert len(iocs['urls']) == 1
#         assert iocs['urls'][0] == url
#     except AssertionError as e:
#         failure = True
#         print('Failed on url: {}'.format(url))

#     if failure:
#         raise AssertionError('Error parsing urls')


# @given(domains())
# @settings(deadline=None)
# def test_domain_parsing(domain):
#     domain = domain.lower()
#     iocs = find_iocs(domain)
#     failure = False

#     try:
#         assert len(iocs['domains']) == 1
#         assert iocs['domains'][0] == domain
#     except AssertionError as e:
#         failure = True
#         print('Failed on domain: {}'.format(domain))

#     if failure:
#         raise AssertionError('Error parsing domains')


# @given(ip_addresses(v=4))
# @settings(deadline=None)
# def test_ipv4_parsing(ipv4):
#     iocs = find_iocs(ipv4)
#     failure = False

#     try:
#         assert len(iocs['ipv4s']) == 1
#         assert iocs['ipv4s'][0] == ipv4
#     except AssertionError as e:
#         failure = True
#         print('Failed on ipv4: {}'.format(ipv4))

#     if failure:
#         raise AssertionError('Error parsing ipv4s')


# @given(ip_addresses(v=6))
# @settings(deadline=None)
# def test_ipv6_parsing(ipv6):
#     iocs = find_iocs(ipv6)
#     failure = False

#     try:
#         assert len(iocs['ipv6s']) == 1
#         assert iocs['ipv6s'][0] == ipv6
#     except AssertionError as e:
#         failure = True
#         print('Failed on ipv6: {}'.format(ipv6))

#     if failure:
#         raise AssertionError('Error parsing ipv6s')



"""Python package for finding observables in text."""

import json
import urllib.parse as urlparse
from typing import Callable, Dict, Iterable, List, Mapping, Union

import click
import ioc_fanger
from d8s_strings import string_remove_from_end
from pyparsing import ParseResults

from ioc_finder import ioc_grammars

IndicatorList = List[str]
IndicatorDict = Dict[str, IndicatorList]
# using `Mapping` b/c it is covariant (https://mypy.readthedocs.io/en/stable/generics.html#variance-of-generic-types)
IndicatorData = Mapping[str, Union[IndicatorList, IndicatorDict]]

DEFAULT_IOC_TYPES = [
    "asns",
    "attack_mitigations",
    "attack_tactics",
    "attack_techniques",
    "authentihashes",
    "bitcoin_addresses",
    "cves",
    "domains",
    "email_addresses",
    "email_addresses_complete",
    "file_paths",
    "google_adsense_publisher_ids",
    "google_analytics_tracker_ids",
    "imphashes",
    "ipv4_cidrs",
    "ipv4s",
    "ipv6s",
    "mac_addresses",
    "md5s",
    "monero_addresses",
    "registry_key_paths",
    "sha1s",
    "sha256s",
    "sha512s",
    "ssdeeps",
    "tlp_labels",
    "urls",
    "user_agents",
    "xmpp_addresses",
]


def _deduplicate(indicator_list: Iterable) -> List:
    """Deduplicate the list of observables."""
    return list(set(indicator_list))


def _listify(indicator_list: ParseResults) -> List:
    """Convert the multi-dimensional list into a one-dimensional list with empty entries and duplicates removed."""
    return _deduplicate([indicator[0] for indicator in indicator_list if indicator[0]])


def _remove_items(items: List[str], text: str) -> str:
    """Remove each item from the text."""
    for item in items:
        text = text.replace(item, " ")
    return text


def _get_items(
    iocs: IndicatorData,
    key: str,
    func_if_none: Callable[[str], IndicatorList],
    text: str,
    **kwargs,
) -> IndicatorList:
    data: IndicatorList = iocs.get(key)  # type: ignore
    if data is None:
        data = func_if_none(text, **kwargs)
    return data


def prepare_text(text: str) -> str:
    """Prepare the text for parsing.

    Currently, this involves fanging (https://ioc-fang.hightower.space/) the text."""
    text = ioc_fanger.fang(text)
    # text = text.encode('idna').decode('utf-8')
    return text


def _clean_url(url: str) -> str:
    """Clean the given URL, removing common, unwanted characters which are usually not part of the URL."""
    # if there is a ")" in the URL and not a "(", remove everything including and after the ")"
    if ")" in url and "(" not in url:
        url = url.split(")")[0]

    # remove `"` and `'` characters from the end of a URL
    url = url.rstrip('"').rstrip("'")

    # remove `'/>` and `"/>` from the end of a URL (this character string occurs at the end of an HMTL tag with )
    url = string_remove_from_end(url, "'/>")
    url = string_remove_from_end(url, '"/>')

    return url


def parse_urls(text: str, *, parse_urls_without_scheme: bool = True) -> List:
    """."""
    if parse_urls_without_scheme:
        url_parse_results = ioc_grammars.scheme_less_url.searchString(text)
    else:
        url_parse_results = ioc_grammars.url.searchString(text)
    urls = _listify(url_parse_results)

    clean_urls = map(_clean_url, urls)

    # I deduplicate them again because the structure of the URL may have changed when it was cleaned
    return _deduplicate(clean_urls)


def _remove_url_domain_name(urls: List, text) -> str:
    """Remove the domain name of each url from the text."""
    for url in urls:
        parsed_url = ioc_grammars.scheme_less_url.parseString(url)
        text = text.replace(parsed_url.url_authority, " ")
    return text


def _remove_url_paths(urls: List, text: str) -> str:
    """Remove the path of each url from the text."""
    for url in urls:
        parsed_url = ioc_grammars.scheme_less_url.parseString(url)
        url_path = urlparse.unquote_plus(parsed_url.url_path)

        is_cidr_range = parse_ipv4_cidrs(str(url))
        # if the 'url' has a URL path and is not a cidr range, remove the url_path
        if not is_cidr_range and len(url_path) > 1:
            text = text.replace(url_path, " ")
    return text


def _percent_decode_url(urls: List, text: str) -> str:
    for url in urls:
        text = text.replace(url, urlparse.unquote_plus(url))
    return text


def parse_domain_names(text):
    """."""
    domains = ioc_grammars.domain_name.searchString(text.lower())
    return _listify(domains)


def parse_ipv4_addresses(text):
    """."""
    addresses = ioc_grammars.ipv4_address.searchString(text)
    return _listify(addresses)


def parse_ipv6_addresses(text):
    """."""
    addresses = ioc_grammars.ipv6_address.searchString(text)
    return _listify(addresses)


def parse_complete_email_addresses(text: str) -> List:
    """."""
    email_addresses = ioc_grammars.complete_email_address.searchString(text)
    return _listify(email_addresses)


def parse_email_addresses(text: str) -> List:
    """."""
    email_addresses = ioc_grammars.email_address.searchString(text)
    return _listify(email_addresses)


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_imphashes_(text: str) -> List:
    """."""
    full_imphash_instances = _listify(ioc_grammars.imphash.searchString(text.lower()))

    imphashes = []

    for imphash in full_imphash_instances:
        imphashes.append(ioc_grammars.imphash.parseString(imphash).hash[0])

    return imphashes


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_authentihashes_(text: str) -> List:
    """."""
    full_authentihash_instances = _listify(ioc_grammars.authentihash.searchString(text.lower()))

    authentihashes = []

    for authentihash in full_authentihash_instances:
        authentihashes.append(ioc_grammars.authentihash.parseString(authentihash).hash[0])

    return authentihashes


def parse_md5s(text):
    """."""
    md5s = ioc_grammars.md5.searchString(text)
    return _listify(md5s)


def parse_sha1s(text):
    """."""
    sha1s = ioc_grammars.sha1.searchString(text)
    return _listify(sha1s)


def parse_sha256s(text):
    """."""
    sha256s = ioc_grammars.sha256.searchString(text)
    return _listify(sha256s)


def parse_sha512s(text):
    """."""
    sha512s = ioc_grammars.sha512.searchString(text)
    return _listify(sha512s)


def parse_ssdeeps(text):
    """."""
    ssdeeps = ioc_grammars.ssdeep.searchString(text)
    return _listify(ssdeeps)


def parse_asns(text):
    """."""
    asns = ioc_grammars.asn.searchString(text)
    return _listify(asns)


def parse_cves(text):
    """."""
    cves = ioc_grammars.cve.searchString(text)
    return _listify(cves)


def parse_ipv4_cidrs(text: str) -> List:
    """."""
    cidrs = ioc_grammars.ipv4_cidr.searchString(text)
    return _listify(cidrs)


def parse_registry_key_paths(text):
    """."""
    parsed_registry_key_paths = ioc_grammars.registry_key_path.searchString(text)
    full_parsed_registry_key_paths = _listify(parsed_registry_key_paths)

    registry_key_paths = []
    for registry_key_path in full_parsed_registry_key_paths:
        # if there is a space in the last section of the parsed registry key path,
        # remove it so that content after a registry key path is not also pulled in...
        # this is a limitation of the grammar:
        # it will not parse a registry key path with a space in the final section (the section after the final '\')
        if " " in registry_key_path.split("\\")[-1]:
            last_section = registry_key_path.split("\\")[-1]
            registry_key_path = registry_key_path.replace(last_section, last_section.split(" ")[0])
            registry_key_paths.append(registry_key_path)
        else:
            registry_key_paths.append(registry_key_path)

    return registry_key_paths


def parse_google_adsense_ids(text):
    """."""
    adsense_publisher_ids = ioc_grammars.google_adsense_publisher_id.searchString(text)
    return _listify(adsense_publisher_ids)


def parse_google_analytics_ids(text):
    """."""
    analytics_tracker_ids = ioc_grammars.google_analytics_tracker_id.searchString(text)
    return _listify(analytics_tracker_ids)


def parse_bitcoin_addresses(text):
    """."""
    bitcoin_addresses = ioc_grammars.bitcoin_address.searchString(text)
    return _listify(bitcoin_addresses)


def parse_monero_addresses(text):
    """."""
    monero_addresses = ioc_grammars.monero_address.searchString(text)
    return _listify(monero_addresses)


def parse_xmpp_addresses(text: str) -> List:
    """."""
    xmpp_addresses = ioc_grammars.xmpp_address.searchString(text)
    return _listify(xmpp_addresses)


def _remove_xmpp_local_part(xmpp_addresses: List, text: str) -> str:
    """Remove the local part of each xmpp_address from the text."""
    for address in xmpp_addresses:
        text = text.replace(address.split("@")[0] + "@", " ")

    return text


def parse_mac_addresses(text):
    """."""
    mac_addresses = ioc_grammars.mac_address.searchString(text)
    return _listify(mac_addresses)


def parse_user_agents(text):
    """."""
    user_agents = ioc_grammars.user_agent.searchString(text)
    return _listify(user_agents)


def parse_file_paths(text):
    """."""
    file_paths = ioc_grammars.file_path.searchString(text)
    return _listify(file_paths)


def parse_pre_attack_tactics(text):
    """."""
    data = ioc_grammars.pre_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_pre_attack_techniques(text):
    """."""
    data = ioc_grammars.pre_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_mitigations(text):
    """."""
    data = ioc_grammars.enterprise_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_tactics(text):
    """."""
    data = ioc_grammars.enterprise_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_techniques(text):
    """."""
    data = ioc_grammars.enterprise_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_mitigations(text):
    """."""
    data = ioc_grammars.mobile_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_tactics(text):
    """."""
    data = ioc_grammars.mobile_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_techniques(text):
    """."""
    data = ioc_grammars.mobile_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_tlp_labels(text):
    """."""
    tlp_labels = ioc_grammars.tlp_label.searchString(text)
    return _listify(tlp_labels)


@click.command()
@click.argument("text", required=False)
@click.option("--no_url_domain_parsing", is_flag=True, help="Using this flag will not parse domain names from URLs")
@click.option(
    "--no_parse_from_url_path", is_flag=True, help="Using this flag will not parse observables from URL paths"
)
@click.option(
    "--no_email_addr_domain_parsing",
    is_flag=True,
    help="Using this flag will not parse domain names from email addresses",
)
@click.option(
    "--no_cidr_address_parsing", is_flag=True, help="Using this flag will not parse IP addresses from CIDR ranges"
)
@click.option(
    "--no_xmpp_addr_domain_parsing",
    is_flag=True,
    help="Using this flag will not parse domain names from XMPP addresses",
)  # pylint: disable=R0913
@click.option(
    "--parse_urls_without_scheme",
    is_flag=True,
    help="Using this flag will parse URLs with and without a scheme (default is True)",
    default=True,
)
@click.option("--no_import_hashes", is_flag=True, help="Using this flag will not parse import hashes")
@click.option("--no_authentihashes", is_flag=True, help="Using this flag will not parse authentihashes")
def cli_find_iocs(
    text,
    no_url_domain_parsing,
    no_parse_from_url_path,
    no_email_addr_domain_parsing,
    no_cidr_address_parsing,
    no_xmpp_addr_domain_parsing,
    parse_urls_without_scheme,
    no_import_hashes,
    no_authentihashes,
):
    """CLI interface for parsing observables."""
    stdin_text = click.get_text_stream("stdin")

    # if there is stdin, use it
    if not text and stdin_text:
        text = "\n".join(stdin_text)
        # text = '\n'.join([line for line in stdin_text])

    iocs = find_iocs(
        text,
        parse_domain_from_url=not no_url_domain_parsing,
        parse_from_url_path=not no_parse_from_url_path,
        parse_domain_from_email_address=not no_email_addr_domain_parsing,
        parse_address_from_cidr=not no_cidr_address_parsing,
        parse_domain_name_from_xmpp_address=not no_xmpp_addr_domain_parsing,
        parse_urls_without_scheme=parse_urls_without_scheme,
        parse_imphashes=not no_import_hashes,
        parse_authentihashes=not no_authentihashes,
    )
    ioc_string = json.dumps(iocs, indent=4, sort_keys=True)
    print(ioc_string)


def find_iocs(  # noqa: CCR001 pylint: disable=R0912,R0915
    text: str,
    *,
    parse_domain_from_url: bool = True,
    parse_from_url_path: bool = True,
    parse_domain_from_email_address: bool = True,
    parse_address_from_cidr: bool = True,
    parse_domain_name_from_xmpp_address: bool = True,
    parse_urls_without_scheme: bool = True,
    parse_imphashes: bool = True,
    parse_authentihashes: bool = True,
    included_ioc_types: List[str] = DEFAULT_IOC_TYPES,
) -> IndicatorData:
    """Find observables (a.k.a. indicators of compromise) in the given text."""
    iocs = {}

    text = prepare_text(text)
    # keep a copy of the original text - some items should be parsed from the original text
    original_text = text

    # urls
    if "urls" in included_ioc_types:
        iocs["urls"] = parse_urls(text, parse_urls_without_scheme=parse_urls_without_scheme)
        if not parse_domain_from_url and not parse_from_url_path:
            text = _remove_items(iocs["urls"], text)
        elif not parse_domain_from_url:
            text = _percent_decode_url(iocs["urls"], text)
            text = _remove_url_domain_name(iocs["urls"], text)
        elif not parse_from_url_path:
            text = _percent_decode_url(iocs["urls"], text)
            text = _remove_url_paths(iocs["urls"], text)
        else:
            text = _percent_decode_url(iocs["urls"], text)

    # xmpp addresses
    if "xmpp_addresses" in included_ioc_types:
        iocs["xmpp_addresses"] = parse_xmpp_addresses(text)

    if "domains" in included_ioc_types and not parse_domain_name_from_xmpp_address:
        xmpp_addresses = _get_items(iocs, "xmpp_addresses", parse_xmpp_addresses, text)
        text = _remove_items(xmpp_addresses, text)
    # even if we want to parse domain names from the xmpp_address,
    # we don't want them also being caught as email addresses so we'll remove everything before the `@`
    elif "email_addresses_complete" in included_ioc_types or "email_addresses" in included_ioc_types:
        xmpp_addresses = _get_items(iocs, "xmpp_addresses", parse_xmpp_addresses, text)
        text = _remove_xmpp_local_part(xmpp_addresses, text)

    # complete email addresses
    if "email_addresses_complete" in included_ioc_types:
        iocs["email_addresses_complete"] = parse_complete_email_addresses(text)
    if "email_addresses" in included_ioc_types:
        iocs["email_addresses"] = parse_email_addresses(text)

    if not parse_domain_from_email_address:
        email_addresses_complete = _get_items(iocs, "email_addresses_complete", parse_complete_email_addresses, text)
        email_addresses = _get_items(iocs, "email_addresses", parse_email_addresses, text)

        text = _remove_items(email_addresses_complete, text)
        text = _remove_items(email_addresses, text)

    if "ipv6s" in included_ioc_types:
        # after parsing the email addresses, we need to remove the
        # '[IPv6:' bit from any of the email addresses so that ipv6 addresses are not extraneously parsed
        text = _remove_items(["[IPv6:"], text)

    # cidr ranges
    if "ipv4_cidrs" in included_ioc_types:
        iocs["ipv4_cidrs"] = parse_ipv4_cidrs(text)

    # remove URLs that are also ipv4_cidrs (see https://github.com/fhightower/ioc-finder/issues/91)
    url_parsing_requires_cidr_removal = "urls" in included_ioc_types and parse_urls_without_scheme
    ip_address_parsing_requires_cidr_removal = "ipv4s" in included_ioc_types and not parse_address_from_cidr
    if url_parsing_requires_cidr_removal or ip_address_parsing_requires_cidr_removal:
        cidr_ranges = _get_items(iocs, "ipv4_cidrs", parse_ipv4_cidrs, text)
        if url_parsing_requires_cidr_removal:
            for cidr in cidr_ranges:
                if cidr in iocs["urls"]:
                    iocs["urls"].remove(cidr)
        if ip_address_parsing_requires_cidr_removal:
            text = _remove_items(cidr_ranges, text)

    # file hashes
    if "imphashes" in included_ioc_types:
        if parse_imphashes:
            iocs["imphashes"] = parse_imphashes_(text)
    if "md5s" in included_ioc_types:
        # remove the imphashes so they are not also parsed as md5s
        imphashes = _get_items(iocs, "imphashes", parse_imphashes_, text)
        text = _remove_items(imphashes, text)

    if "authentihashes" in included_ioc_types:
        if parse_authentihashes:
            iocs["authentihashes"] = parse_authentihashes_(text)
    if "sha256s" in included_ioc_types:
        # remove the authentihashes so they are not also parsed as sha256s
        authentihashes = _get_items(iocs, "authentihashes", parse_authentihashes_, text)
        text = _remove_items(authentihashes, text)

    # domains
    if "domains" in included_ioc_types:
        iocs["domains"] = parse_domain_names(text)

    # ip addresses
    if "ipv4s" in included_ioc_types:
        iocs["ipv4s"] = parse_ipv4_addresses(text)
    if "ipv6s" in included_ioc_types:
        iocs["ipv6s"] = parse_ipv6_addresses(text)

    # file hashes
    if "sha512s" in included_ioc_types:
        iocs["sha512s"] = parse_sha512s(text)
    if "sha256s" in included_ioc_types:
        iocs["sha256s"] = parse_sha256s(text)
    if "sha1s" in included_ioc_types:
        iocs["sha1s"] = parse_sha1s(text)
    if "md5s" in included_ioc_types:
        iocs["md5s"] = parse_md5s(text)
    if "ssdeeps" in included_ioc_types:
        iocs["ssdeeps"] = parse_ssdeeps(text)

    # misc
    if "asns" in included_ioc_types:
        iocs["asns"] = parse_asns(text)
    if "cves" in included_ioc_types:
        iocs["cves"] = parse_cves(original_text)
    if "registry_key_paths" in included_ioc_types:
        iocs["registry_key_paths"] = parse_registry_key_paths(text)
    if "google_adsense_publisher_ids" in included_ioc_types:
        iocs["google_adsense_publisher_ids"] = parse_google_adsense_ids(text)
    if "google_analytics_tracker_ids" in included_ioc_types:
        iocs["google_analytics_tracker_ids"] = parse_google_analytics_ids(text)
    if "bitcoin_addresses" in included_ioc_types:
        iocs["bitcoin_addresses"] = parse_bitcoin_addresses(text)
    if "monero_addresses" in included_ioc_types:
        iocs["monero_addresses"] = parse_monero_addresses(text)
    if "mac_addresses" in included_ioc_types:
        iocs["mac_addresses"] = parse_mac_addresses(text)
    if "user_agents" in included_ioc_types:
        iocs["user_agents"] = parse_user_agents(text)
    if "tlp_labels" in included_ioc_types:
        iocs["tlp_labels"] = parse_tlp_labels(original_text)

    if "attack_mitigations" in included_ioc_types:
        iocs["attack_mitigations"] = {  # type: ignore
            "enterprise": parse_enterprise_attack_mitigations(original_text),
            "mobile": parse_mobile_attack_mitigations(original_text),
        }

    if "attack_tactics" in included_ioc_types:
        iocs["attack_tactics"] = {  # type: ignore
            "pre_attack": parse_pre_attack_tactics(original_text),
            "enterprise": parse_enterprise_attack_tactics(original_text),
            "mobile": parse_mobile_attack_tactics(original_text),
        }

    if "attack_techniques" in included_ioc_types:
        iocs["attack_techniques"] = {  # type: ignore
            "pre_attack": parse_pre_attack_techniques(original_text),
            "enterprise": parse_enterprise_attack_techniques(original_text),
            "mobile": parse_mobile_attack_techniques(original_text),
        }

    if "file_paths" in included_ioc_types:
        # if there are still url paths in the text, remove them so they don't get parsed as file names
        if parse_from_url_path:
            urls = _get_items(iocs, "urls", parse_urls, text, parse_urls_without_scheme=parse_urls_without_scheme)
            text = _remove_url_paths(urls, text)

        iocs["file_paths"] = parse_file_paths(text)

    return iocs



from pytest import param

TLP_DATA = [
    param("tlp amber and TLP:RED", {"tlp_labels": ["TLP:RED", "TLP:AMBER"]}, {}, id="tlp_1"),
    param("tlp-Amber and TLPRED TlpGreen", {"tlp_labels": ["TLP:RED", "TLP:AMBER", "TLP:GREEN"]}, {}, id="tlp_2"),
]



"""These tests make sure the included_ioc_types parameter is working properly.

Each test below passes a string with two IOC types into the find_iocs function, but only specifies one `included_ioc_types` argument to ensure it is handled properly."""

from pytest import param

from ioc_finder.ioc_finder import DEFAULT_IOC_TYPES

IOC_EXAMPLES = {
    "domains": ["abc.py", "bar.com", "example.com", "foo.com", "swissjabber.de"],
    "urls": ["https://example.com/test%20page/foo.com/bingo.php?q=bar.com"],
    "xmpp_addresses": ["foo@swissjabber.de"],
    "email_addresses_complete": ["me@example.com"],
    "email_addresses": ["me@example.com"],
    "ipv4_cidrs": ["1.1.1.1/0"],
    "imphashes": ["18ddf28a71089acdbab5038f58044c0a"],
    "authentihashes": ["3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4"],
    "ipv4s": ["1.1.1.1"],
    "ipv6s": ["2001:0db8:0000:0000:0000:ff00:0042:8329"],
    "sha512s": ["a" * 128],
    "sha256s": ["a" * 64],
    "sha1s": ["a" * 40],
    "md5s": ["a" * 32],
    "ssdeeps": [
        "12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F",
        "0000:0000:ff00",
        "2001:0db8:0000",
    ],  # I don't like that the components of an ipv6 can be parsed as an ssdeep... I've ticketed this here: https://github.com/fhightower/ioc-finder/issues/228
    "asns": ["ASN123"],
    "cves": ["CVE-2022-1234"],
    "registry_key_paths": [r"HKEY_LOCAL_MACHINE\Software\Microsoft\Windows"],
    "google_adsense_publisher_ids": ["pub-1234567891234567"],
    "google_analytics_tracker_ids": ["UA-000000-1"],
    "bitcoin_addresses": ["18ddf28a71089acdbab5038f58044c0a", "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy"],
    "monero_addresses": [
        "496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp"
    ],
    "mac_addresses": ["AA-F2-C9-A6-B3-4F"],
    "user_agents": [
        "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1) TLP"
    ],  # I don't like this parsing... I've ticketed this for improvement here: https://github.com/fhightower/ioc-finder/issues/227
    "tlp_labels": ["TLP:RED"],
    "mac_addresses": ["AA-F2-C9-A6-B3-4F"],
    "file_paths": ["~/foo/bar/abc.py"],
    "attack_mitigations": {"enterprise": ["M1036", "M1015"]},
    "attack_tactics": {"pre_attack": ["TA0012"]},
    "attack_techniques": {"pre_attack": ["T1329"]},
}
all_ioc_text = " ".join([val for sublist in IOC_EXAMPLES.values() for val in sublist])

# this is a hack to be fixed in https://github.com/fhightower/ioc-finder/issues/224
# imphashes and authentihashes require the hash to be prefixed with `imphash` and `authentihash` respectively, but when parsed, only the hash itself will be present
all_ioc_text = all_ioc_text.replace(IOC_EXAMPLES["imphashes"][0], f'imphash {IOC_EXAMPLES["imphashes"][0]}')  # type: ignore
all_ioc_text = all_ioc_text.replace(
    IOC_EXAMPLES["authentihashes"][0], f'authentihash {IOC_EXAMPLES["authentihashes"][0]}'  # type: ignore
)
all_ioc_text = all_ioc_text.replace(IOC_EXAMPLES["user_agents"][0], IOC_EXAMPLES["user_agents"][0].rstrip(" TLP"))  # type: ignore
# add the attack data
all_ioc_text = all_ioc_text + " " + " ".join(IOC_EXAMPLES["attack_mitigations"]["enterprise"])  # type: ignore
all_ioc_text = all_ioc_text + " " + " ".join(IOC_EXAMPLES["attack_tactics"]["pre_attack"])  # type: ignore
all_ioc_text = all_ioc_text + " " + " ".join(IOC_EXAMPLES["attack_techniques"]["pre_attack"])  # type: ignore


individual_included_ioc_types_tests = []

for type_ in DEFAULT_IOC_TYPES:
    individual_included_ioc_types_tests.append(
        param(
            all_ioc_text,
            {type_: IOC_EXAMPLES[type_]},
            {"included_ioc_types": [type_]},
            id=f"Only find {type_} with included_ioc_types",
        )
    )


multiple_included_ioc_types_tests = []

# make sure multiple included_ioc_types are handled properly
multiple_included_ioc_types_tests.append(
    param(
        "https://example.com/test%20page/foo.com/bingo.php?q=bar.com",
        {
            "domains": ["bar.com", "example.com", "foo.com"],
            "urls": ["https://example.com/test%20page/foo.com/bingo.php?q=bar.com"],
        },
        {"included_ioc_types": ["domains", "urls"]},
        id="Find multiple data types",
    )
)

# make sure multiple included_ioc_types work well with other kwargs - the list of domains is missing `foo.com` b/c `parse_from_url_path` is False
multiple_included_ioc_types_tests.append(
    param(
        "https://example.com/test%20page/foo.com/bingo.php?q=bar.com",
        {
            "domains": ["bar.com", "example.com"],
            "urls": ["https://example.com/test%20page/foo.com/bingo.php?q=bar.com"],
        },
        {"included_ioc_types": ["domains", "urls"], "parse_from_url_path": False},
        id="Find multiple data types",
    )
)



from .asns import ASN_DATA
from .attack_data import ATTACK_DATA
from .coins import COIN_DATA
from .cves import CVE_DATA
from .domains import DOMAIN_DATA
from .email import EMAIL_DATA
from .feature__included_ioc_types import individual_included_ioc_types_tests, multiple_included_ioc_types_tests
from .file_paths import PATH_DATA
from .hashes import HASH_DATA
from .ids import ID_DATA
from .ip_addr import IP_DATA
from .mac_addr import MAC_DATA
from .registry_keys import REGISTRY_DATA
from .tlp_labels import TLP_DATA
from .urls import URL_DATA
from .user_agents import UA_DATA

cases = [
    TLP_DATA,
    individual_included_ioc_types_tests,
    multiple_included_ioc_types_tests,
    DOMAIN_DATA,
    EMAIL_DATA,
    HASH_DATA,
    CVE_DATA,
    IP_DATA,
    ASN_DATA,
    ATTACK_DATA,
    REGISTRY_DATA,
    PATH_DATA,
    ID_DATA,
    COIN_DATA,
    MAC_DATA,
    URL_DATA,
    UA_DATA,
]

ALL_TESTS = [val for sublist in cases for val in sublist]



from pytest import param

HASH_DATA = [
    param(
        "{} {} {} {} {}".format("A" * 32, "a" * 32, "b" * 40, "c" * 64, "d" * 128),
        {
            "md5s": ["aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"],
            "sha1s": ["bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb"],
            "sha256s": ["cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc"],
            "sha512s": [
                "dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd"
            ],
        },
        {},
        id="random_hash_1",
    ),
    param(
        "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U",
        {"ssdeeps": ["1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"]},
        {},
        id="ssdeep_1",
    ),
    param(
        "ahdfadsfa 1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,000/000/000000001 adfasf",
        {"ssdeeps": ["1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"]},
        {},
        id="ssdeep_2",
    ),
    param(
        """c2b257868686c861d43c6cf3de146b8812778c8283f7d
        Threat  Zepakab/Zebrocy Downloader
        ssdeep  12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F""",
        {"ssdeeps": ["12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F"]},
        {},
        id="ssdeep_3",
    ),
    param(
        """393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b,"000/000/000000001"
        196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg,"000/000/000000002"
        98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S,"000/000/000000003"
        96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg,"000/000/000000004"
        196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax,"000/000/000000005"
        1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6,"000/000/000000006"
        48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ,"000/000/000000007"
        24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ,"000/000/000000008"
        1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,"000/000/000000009""",
        {
            "ssdeeps": [
                "393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b",
                "196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg",
                "98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S",
                "96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg",
                "1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6",
                "48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ",
                "24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ",
                "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U",
                "196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax",
            ]
        },
        {},
        id="ssdeep_4",
    ),
    param(
        """SHA-256 093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a
        Dropper imphash: 18ddf28a71089acdbab5038f58044c0a
        C2 IP: 210.209.127.8:443
        imphash: 18ddf28a71089acdbab5038f58044c0a
        imphash 18ddf28a71089acdbab5038f58044c0a
        imphash  18ddf28a71089acdbab5038f58044c0a
        imphash:     18ddf28a71089acdbab5038f58044c0a
        imphash\t18ddf28a71089acdbab5038f58044c0a
        imphash\n18ddf28a71089acdbab5038f58044c0a
        imphash - 18ddf28a71089acdbab5038f58044c0a""",
        {
            "imphashes": [
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
            ],
            "ipv4s": ["210.209.127.8"],
            "sha256s": ["093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a"],
        },
        {},
        id="imphash_1",
    ),
    param(
        """SHA-256 093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a
        Dropper import hash: 18ddf28a71089acdbab5038f58044c0a
        C2 IP: 210.209.127.8:443
        import hash: 18ddf28a71089acdbab5038f58044c0a
        import hash 18ddf28a71089acdbab5038f58044c0a
        import hash  18ddf28a71089acdbab5038f58044c0a
        import hash:     18ddf28a71089acdbab5038f58044c0a
        import hash\t18ddf28a71089acdbab5038f58044c0a
        import hash\n18ddf28a71089acdbab5038f58044c0a
        import hash - 18ddf28a71089acdbab5038f58044c0a""",
        {
            "imphashes": [
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
            ],
            "ipv4s": ["210.209.127.8"],
            "sha256s": ["093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a"],
        },
        {},
        id="imphash_3",
    ),
    param(
        """SHA-256 093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a
        Dropper IMPORT HASH: 18ddf28a71089acdbab5038f58044c0a
        C2 IP: 210.209.127.8:443
        IMPORT HASH: 18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH 18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH  18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH:     18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH\t18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH\n18ddf28a71089acdbab5038f58044c0a
        IMPORT HASH - 18ddf28a71089acdbab5038f58044c0a""",
        {
            "imphashes": [
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
                "18ddf28a71089acdbab5038f58044c0a",
            ],
            "ipv4s": ["210.209.127.8"],
            "sha256s": ["093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a"],
        },
        {},
        id="imphash_4",
    ),
    param(
        """
        authentihash 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash   3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash: 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash:     3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash - 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash-3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash\t3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        authentihash\n3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        """,
        {
            "authentihashes": [
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
            ]
        },
        {},
        id="authentihash_1",
    ),
    param(
        """
        AUTHENTIHASH 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH   3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH: 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH:     3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH - 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH-3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH\t3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        AUTHENTIHASH\n3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        """,
        {
            "authentihashes": [
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
                "3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4",
            ]
        },
        {},
        id="authentihash_2",
    ),
]



from pytest import param

PATH_DATA = [
    param(
        """C:\\Users\\<username>\\AppData \\Local\\Microsoft\\Windows\\shedaudio.exe

        C:\\Users\\<username>\\AppData\\Roaming\\Macromedia\\Flash Player\\macromedia\\bin\\flashplayer.exe

        Typical Registry Keys:

        HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Run

        HKEY_LOCAL_MACHINE\\Software\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Run

        HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run

        System Root Directories:

        C:\\Windows\\11987416.exe

        C:\\Windows\\System32\\46615275.exe

        C:\\Windows\\System32\\shedaudio.exe

        C:\\Windows\\SysWOW64\\f9jwqSbS.exe""",
        {
            "file_paths": [
                "C:\\Users\\<username>\\AppData \\Local\\Microsoft\\Windows\\shedaudio.exe",
                "C:\\Users\\<username>\\AppData\\Roaming\\Macromedia\\Flash Player\\macromedia\\bin\\flashplayer.exe",
                "C:\\Windows\\11987416.exe",
                "C:\\Windows\\System32\\46615275.exe",
                "C:\\Windows\\System32\\shedaudio.exe",
                "C:\\Windows\\SysWOW64\\f9jwqSbS.exe",
            ],
            "registry_key_paths": [
                "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run",
                "HKEY_LOCAL_MACHINE\\Software\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Run",
                "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Run",
            ],
        },
        {},
        id="file_path_1",
    ),
    param(
        "/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex",
        {"file_paths": ["/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex"]},
        {},
        id="file_path_2",
    ),
    param(
        "and this is a file ~/foo/bar/abc.py",
        {"file_paths": ["~/foo/bar/abc.py"], "domains": ["abc.py"]},
        {},
        id="file_path_3",
    ),
    param(
        "test /Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex file",
        {"file_paths": ["/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex"]},
        {},
        id="file_path_4",
    ),
    param(
        "another home directory ~/Desktop/test.py python file",
        {"file_paths": ["~/Desktop/test.py"], "domains": ["test.py"]},
        {},
        id="file_path_5",
    ),
    # param(
    #     "/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex",
    #     {'file_paths': ["/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex"]},
    #     {},
    #     id="file_path_2"
    # )
]



import concurrent.futures

from ioc_finder import find_iocs


def test_nested_concurrency():
    texts = ["example.com", "foo bar bang buzz", "This is just an example.com https://example.org/test/bingo.php"]
    results = []

    with concurrent.futures.ThreadPoolExecutor() as executor:
        results = [i for i in executor.map(find_iocs, texts)]

    assert results[0]["domains"] == ["example.com"]
    assert "example.com" in results[2]["domains"]
    assert "example.org" in results[2]["domains"]
    assert results[2]["urls"] == ["https://example.org/test/bingo.php"]



from typing import Dict

import pytest

from ioc_finder import find_iocs
from ioc_finder.ioc_finder import IndicatorDict, IndicatorList

from .find_iocs_cases import ALL_TESTS


@pytest.mark.parametrize("text, results, args", ALL_TESTS)
def test_find_iocs(text: str, results: Dict, args: Dict) -> None:
    # Parse input
    iocs = find_iocs(text, **args)

    for key, value in iocs.items():
        # Compare lists
        if isinstance(value, list):
            _compare_lists(key, value, results.get(key, []))
        # Compare sub dictionaries like for attack patterns
        elif isinstance(value, dict):
            _compare_dicts(key, value, results.get(key, {}))


def _compare_lists(key_name: str, ioc_list: IndicatorList, result_list: IndicatorList) -> None:
    assert sorted(ioc_list) == sorted(
        result_list
    ), f"Unexpected result for key '[{key_name}]' -> Expected: '{result_list}' Received: '{ioc_list}'"


def _compare_dicts(key_name: str, ioc_dict: IndicatorDict, result_dict: IndicatorDict) -> None:
    for ioc_key, ioc_value in ioc_dict.items():
        if isinstance(ioc_value, list):
            _compare_lists(f"{key_name},{ioc_key}", ioc_value, result_dict.get(ioc_key, []))



import copy
import re

from pyparsing import (
    CaselessLiteral,
    Char,
    Combine,
    Empty,
    FollowedBy,
    Literal,
    MatchFirst,
    NotAny,
    OneOrMore,
    Optional,
    Or,
    Regex,
    Word,
    WordEnd,
    WordStart,
    ZeroOrMore,
    alphanums,
    alphas,
    hexnums,
    nums,
    one_of,
    printables,
    pyparsing_common,
    replace_with,
)

from ioc_finder.data import (
    enterprise_attack_mitigations,
    enterprise_attack_tactics,
    enterprise_attack_techniques,
    mobile_attack_mitigations,
    mobile_attack_tactics,
    mobile_attack_techniques,
    pre_attack_tactics,
    pre_attack_techniques,
    schemes,
    tlds,
)

alphanum_word_start = WordStart(wordChars=alphanums)
alphanum_word_end = WordEnd(wordChars=alphanums)

# the label definition ignores the fact that labels should not end in an hyphen
label = Word(initChars=alphanums + "_", bodyChars=alphanums + "-_", max=63)
domain_tld = one_of(tlds, caseless=True)
domain_name = (
    alphanum_word_start
    + Combine(
        Combine(OneOrMore(label + ("." + FollowedBy(Word(alphanums + "-_")))))("domain_labels") + domain_tld("tld")
    )
    + alphanum_word_end
).set_parse_action(pyparsing_common.downcase_tokens)

ipv4_section = (
    Word(nums, asKeyword=True, max=3)
    .set_parse_action(lambda x: str(int(x[0])))
    .addCondition(lambda tokens: int(tokens[0]) < 256)
)
# basically, the grammar below says: start any words that start with a '.' or a number;
# I want to match words that start with a '.' because this will fail later in the grammar...
# and I do not want to match anything that start with a '.'
ipv4_address = (
    alphanum_word_start
    + WordStart("." + nums)
    + Combine((ipv4_section + ".") * 3 + ipv4_section)
    + NotAny(Regex(r"\.\S"))
    + alphanum_word_end
)

ipv6_word_start = WordStart(wordChars=alphanums + ":")
ipv6_word_end = WordEnd(wordChars=alphanums + ":")

hexadectet = Word(hexnums, min=1, max=4)
ipv6_address_full = ipv6_word_start + Combine((hexadectet + ":") * 7 + hexadectet)

# the condition on the end of this grammar is designed to make sure that any shortened ipv6 addresses have '::' in them
ipv6_address_shortened = Combine(OneOrMore(Or([hexadectet + Word(":"), Word(":")])) + hexadectet).addCondition(
    lambda tokens: tokens[0].count("::") > 0
)

ipv6_address = (
    Or([ipv6_address_full, ipv6_address_shortened]).addCondition(lambda tokens: tokens[0].count(":") > 1)
    + ipv6_word_end
)

complete_email_comment = Regex(r"\([a-zA-Z0-9]+\)")
# the complete_email_local_part grammar ignores the fact that characters like <<<(),:;<>@[\] >>>
# are possible in a quoted complete_email_local_part
# (and the double-quotes and backslash should be preceded by a backslash)
complete_email_local_part = Combine(
    Optional(complete_email_comment)("email_address_start_comment")
    + OneOrMore(MatchFirst([Word(alphanums + "!#$%&'*+-/=?^_`{|}~." + '"'), CaselessLiteral("\\@")]))
    + Optional(complete_email_comment)("email_address_end_comment")
)
complete_email_address = Combine(
    complete_email_local_part("email_address_local_part")
    + "@"
    + Or([domain_name, "[" + ipv4_address + "]", "[IPv6:" + ipv6_address + "]"])("email_address_domain")
)

email_local_part = Word(alphanums, bodyChars=alphanums + "+-_.").set_parse_action(pyparsing_common.downcase_tokens)
email_address = alphanum_word_start + Combine(
    email_local_part("email_address_local_part")
    + "@"
    + Or([domain_name, "[" + ipv4_address + "]", "[IPv6:" + ipv6_address + "]"])("email_address_domain")
)

url_scheme = one_of(schemes, caseless=True)
port = Word(":", nums, min=2)
url_authority = Combine(Or([email_address, domain_name, ipv4_address, ipv6_address]) + Optional(port)("port"))
# The url_path_word characters are taken from https://www.ietf.org/rfc/rfc3986.txt...
# (of particular interest is "Appendix A.  Collected ABNF for URI")
# Although the ":" character is not valid in url paths,
# some urls are written with the ":" unencoded so we include it below
url_path_word = Word(alphanums + "-._~!$&'()*+,;=:%")
url_path = Combine(OneOrMore(MatchFirst([url_path_word, Literal("/")])))
url_query = Word(printables, excludeChars="#\"']")
url_fragment = Word(printables, excludeChars="?\"']")
url = alphanum_word_start + Combine(
    url_scheme("url_scheme")
    + "://"
    + url_authority("url_authority")
    + Optional(Combine("/" + Optional(url_path)))("url_path")
    + (Optional(Combine("?" + url_query)("url_query")) & Optional(Combine("#" + url_fragment)("url_fragment")))
)
scheme_less_url = alphanum_word_start + Or(
    [
        url,
        Combine(
            Combine(url_authority("url_authority") + Combine("/" + Optional(url_path))("url_path"))
            + (Optional(Combine("?" + url_query)("url_query")) & Optional(Combine("#" + url_fragment)("url_fragment")))
        ),
    ]
)

# this allows for matching file hashes preceeded with an 'x' or 'X'...
# see https://github.com/fhightower/ioc-finder/issues/41
file_hash_word_start = WordStart(wordChars=alphanums.replace("x", "").replace("X", ""))
md5 = (
    file_hash_word_start
    + Word(hexnums, exact=32).set_parse_action(pyparsing_common.downcase_tokens)
    + alphanum_word_end
)
imphash = Combine(
    Or([CaselessLiteral("imphash"), CaselessLiteral("import hash")])
    + Optional(Word(printables, excludeChars=alphanums))
    + md5("hash"),
    joinString=" ",
    adjacent=False,
)
sha1 = (
    file_hash_word_start
    + Word(hexnums, exact=40).set_parse_action(pyparsing_common.downcase_tokens)
    + alphanum_word_end
)
sha256 = (
    file_hash_word_start
    + Word(hexnums, exact=64).set_parse_action(pyparsing_common.downcase_tokens)
    + alphanum_word_end
)
authentihash = Combine(
    CaselessLiteral("authentihash") + Optional(Word(printables, excludeChars=alphanums)) + sha256("hash"),
    joinString=" ",
    adjacent=False,
)
sha512 = (
    file_hash_word_start
    + Word(hexnums, exact=128).set_parse_action(pyparsing_common.downcase_tokens)
    + alphanum_word_end
)

year = Word("12") + Word(nums, exact=3)
cve = (
    alphanum_word_start
    + Combine(
        CaselessLiteral("cve").set_parse_action(replace_with("CVE"))
        + Word("- ").set_parse_action(replace_with("-"))
        + year("year")
        + Word("-")
        + Word(nums, min=4)("cve_id")
    )
    + alphanum_word_end
)

asn = (
    alphanum_word_start
    + Combine(
        Or(
            [
                Literal("AS") + Optional(Word("N ")).set_parse_action(replace_with("N")),
                Literal("as").set_parse_action(replace_with("ASN")),
                (Literal("asn") + Optional(" ")).set_parse_action(replace_with("ASN")),
            ]
        )
        + Word(nums)("as_number")
    )
    + alphanum_word_end
)

ipv4_cidr = (
    alphanum_word_start
    + Combine(ipv4_address("cidr_address") + "/" + Word(nums, max=2)("cidr_bit_range"))
    + alphanum_word_end
)

root_key_list = [
    "HKEY_LOCAL_MACHINE",
    "HKLM",
    "HKEY_CURRENT_CONFIG",
    "HKCC",
    "HKEY_CLASSES_ROOT",
    "HKCR",
    "HKEY_CURRENT_USER",
    "HKCU",
    "HKEY_USERS",
    "HKU",
    "HKEY_PERFORMANCE_DATA",
    "HKEY_DYN_DATA",
]
root_key = one_of(root_key_list)


def hasMultipleConsecutiveSpaces(string):
    """Return True if the given string has multiple, consecutive spaces."""
    return re.match("  +", string)


def hasBothOrNeitherAngleBrackets(string):
    """Make sure a string either has both '<' and '>' or neither of those angle brackets."""
    left_angle_bracket_in_string = "<" in string
    right_angle_bracket_in_string = ">" in string

    # if the string has both brackets...
    if left_angle_bracket_in_string and right_angle_bracket_in_string:
        return True
    # if the string has only one bracket...
    elif left_angle_bracket_in_string or right_angle_bracket_in_string:
        return False
    # if the string has neither of the brackets...
    else:
        return True


registry_key_subpath_section = Combine(
    Word("\\")
    + Optional(Word("<"))
    + Word(alphanums)
    + ZeroOrMore(
        # registry key paths may contain a file extension which requires that we capture registry
        # key path sections with a period (e.g. `notepad.exe`)
        Optional(Word(".", max=1))
        # the registry key path section can contain any alphanum text (including spaces) as long as the text is not
        # one of the registry key path root keys and as long as there are not multiple, consecutive spaces
        + Word(alphanums + " ").addCondition(
            lambda tokens: tokens[0].strip() not in root_key_list and not hasMultipleConsecutiveSpaces(tokens[0])
        )
    )
    + Optional(Word(">"))
).addCondition(lambda tokens: hasBothOrNeitherAngleBrackets(tokens[0]))
registry_key_subpath = OneOrMore(registry_key_subpath_section)
registry_key_path = (
    alphanum_word_start
    + Combine(
        Optional("<").set_parse_action(replace_with(""))
        + root_key("registry_key_root")
        + Optional(">").set_parse_action(replace_with(""))
        + registry_key_subpath("registry_key_subpath")
    )
    + alphanum_word_end
)

# see https://support.google.com/adsense/answer/2923881?hl=en
google_adsense_publisher_id = (
    alphanum_word_start
    # we use `Or([Literal("pub-")...` instead of something like `CaselessLiteral("pub-")` b/c...
    # we only want to parse "pub" when it is all upper or lowercased (not "pUb" or other, similar variations)
    + Combine(one_of("pub- PUB-") + Word(nums, exact=16)).set_parse_action(pyparsing_common.downcase_tokens)
    + alphanum_word_end
)

# see https://support.google.com/analytics/answer/7372977?hl=en
google_analytics_tracker_id = (
    alphanum_word_start
    + Combine(
        # we use `Or([Literal("ua-")...` instead of something like `CaselessLiteral("ua-")` b/c...
        # we only want to parse "ua" when it is all upper or lowercased (not "uA" or other, similar variations)
        one_of("ua- UA-")
        + Word(nums, min=6)("account_number")
        + "-"
        + Word(nums)("property_number")
    ).set_parse_action(pyparsing_common.upcase_tokens)
    + alphanum_word_end
)

# see https://en.bitcoin.it/wiki/Address
# (and https://github.com/bitcoin/bips/blob/master/bip-0173.mediawiki#segwit-address-format for Bech32 addresses)
bitcoin_address = (
    alphanum_word_start
    + MatchFirst(
        [
            Regex(r"1[a-zA-Z0-9]{25,34}"),
            Regex(r"3[a-zA-Z0-9]{25,34}"),
            Regex(r"bc1[a-zA-Z0-9]{11,71}"),
        ]
    )
    + alphanum_word_end
)

monero_address = alphanum_word_start + Regex("4[0-9AB][1-9A-HJ-NP-Za-km-z]{93}") + alphanum_word_end

# see https://github.com/fhightower/ioc-finder/issues/18
xmpp_address = alphanum_word_start + Combine(
    email_local_part("email_address_local_part") + "@" + domain_name("jabber_address_domain")
).addCondition(lambda tokens: "jabber" in tokens[0].split("@")[-1] or "xmpp" in tokens[0].split("@")[-1])

# the mac address grammar was developed from https://en.wikipedia.org/wiki/MAC_address#Notational_conventions
# handles xx:xx:xx:xx:xx:xx or xx-xx-xx-xx-xx-xx
mac_address_16_bit_section = Combine((Word(hexnums, exact=2) + one_of("- :")) * 5 + Word(hexnums, exact=2))
# handles xxxx.xxxx.xxxx
mac_address_32_bit_section = Combine((Word(hexnums, exact=4) + ".") * 2 + Word(hexnums, exact=4))
mac_address_word_start = WordStart(wordChars=alphanums + ":-.")
mac_address_word_end = WordEnd(wordChars=alphanums + ":-.")
mac_address = (
    mac_address_word_start + MatchFirst([mac_address_16_bit_section, mac_address_32_bit_section]) + mac_address_word_end
)

# the structure of an ssdeep hash is: chunksize:chunk:double_chunk
# we add a condition to the ssdeep grammar to make sure that the second section of the grammar
# (the chunk) is at least as big if not bigger than the third section (the double_chunk)
ssdeep = alphanum_word_start + Combine(
    Word(nums) + ":" + Word(alphanums + "/+", min=3) + ":" + Word(alphanums + "/+", min=3)
).addCondition(lambda tokens: len(tokens[0].split(":")[1]) >= len(tokens[0].split(":")[2]))

user_agent_platform_version = Regex(r"[0-9]+(\.[0-9]*)*")
user_agent_start = Combine(Regex(r"[Mm]ozilla/") + user_agent_platform_version)
user_agent_details = Regex(r"\(.+?\)")
user_agent_platform = Combine(
    alphanum_word_start
    + Regex(r"[a-zA-Z]{2,}/?").addCondition(lambda tokens: tokens[0].lower().strip("/") != "mozilla")
    + Optional(user_agent_platform_version)
)
user_agent = Combine(
    user_agent_start + user_agent_details + ZeroOrMore(user_agent_platform + Optional(user_agent_details)),
    joinString=" ",
    adjacent=False,
)

# https://github.com/fhightower/ioc-finder/issues/13
file_ending = Word(alphas, max=5)
windows_file_path = alphanum_word_start + Combine(
    Char(alphanums) + ":" + Word(printables + " ", exclude_chars=".") + "." + file_ending
)

# we need to add '/' and '~' to the alphanum_word_start so that the grammar will match words starting with '/' and '~'
# we add ':' to the alphanum_word_start because we want to avoid parsing urls are file paths
# (e.g. "//twitter.com" from "https://twitter.com/")
unix_file_path_wordstart = copy.deepcopy(alphanum_word_start)
unix_file_path_wordstart.wordChars.add(":")
unix_file_path_wordstart.wordChars.add("/")
unix_file_path_wordstart.wordChars.add("~")

unix_file_path = unix_file_path_wordstart + Combine(
    one_of("~ /") + Word(printables + " ", exclude_chars=".") + "." + file_ending
).addCondition(lambda tokens: "//" not in tokens[0])
file_path = Or([windows_file_path, unix_file_path]) + alphanum_word_end

# be aware that the phone_number grammar assumes that the text being sent to it has been reversed
phone_number_connector = Word(" .-", max=3)
phone_number_format_1 = Combine(
    Word(nums, exact=4)
    + phone_number_connector
    + Word(nums, exact=3)
    + Optional(phone_number_connector + Optional(")") + Word(nums) + Optional("("))
)

phone_number = Or([phone_number_format_1])

attack_sub_technique = Literal(".") + Word(nums, exact=3)
pre_attack_tactics_grammar = (
    alphanum_word_start
    + Or([CaselessLiteral(i) for i in pre_attack_tactics]).set_parse_action(pyparsing_common.upcase_tokens)
    + alphanum_word_end
)
pre_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        one_of(pre_attack_techniques, caseless=True).set_parse_action(pyparsing_common.upcase_tokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

enterprise_attack_mitigations_grammar = (
    alphanum_word_start + one_of(enterprise_attack_mitigations, caseless=True) + alphanum_word_end
)
enterprise_attack_tactics_grammar = (
    alphanum_word_start
    + one_of(enterprise_attack_tactics, caseless=True).set_parse_action(pyparsing_common.upcase_tokens)
    + alphanum_word_end
)
enterprise_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        one_of(enterprise_attack_techniques, caseless=True).set_parse_action(pyparsing_common.upcase_tokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

mobile_attack_mitigations_grammar = (
    alphanum_word_start + one_of(mobile_attack_mitigations, caseless=True) + alphanum_word_end
)
mobile_attack_tactics_grammar = (
    alphanum_word_start
    + one_of(mobile_attack_tactics, caseless=True).set_parse_action(pyparsing_common.upcase_tokens)
    + alphanum_word_end
)
mobile_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        one_of(mobile_attack_techniques, caseless=True).set_parse_action(pyparsing_common.upcase_tokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

tlp_colors = one_of("red amber green white", caseless=True)

tlp_label = Combine(
    CaselessLiteral("tlp")
    + Or([Literal(":"), Literal("-"), Literal(" "), Empty()]).set_parse_action(lambda x: ":")
    + tlp_colors
).set_parse_action(pyparsing_common.upcase_tokens)



#!/usr/bin/env python3

from ioc_finder.ioc_grammars import hasBothOrNeitherAngleBrackets, hasMultipleConsecutiveSpaces


def test_hasBothOrNeitherAngleBrackets_1():
    assert hasBothOrNeitherAngleBrackets("<>")
    assert hasBothOrNeitherAngleBrackets("<foo>")
    assert hasBothOrNeitherAngleBrackets("< foo >")
    assert hasBothOrNeitherAngleBrackets("foo")

    assert not hasBothOrNeitherAngleBrackets("<")
    assert not hasBothOrNeitherAngleBrackets("<foo")
    assert not hasBothOrNeitherAngleBrackets("foo<")
    assert not hasBothOrNeitherAngleBrackets("<foo<")
    assert not hasBothOrNeitherAngleBrackets(">")
    assert not hasBothOrNeitherAngleBrackets(">foo")
    assert not hasBothOrNeitherAngleBrackets("foo>")
    assert not hasBothOrNeitherAngleBrackets(">foo>")


def test_hasMultipleConsecutiveSpaces_1():
    assert not hasMultipleConsecutiveSpaces("")
    assert not hasMultipleConsecutiveSpaces(" ")
    assert hasMultipleConsecutiveSpaces("  ")
    assert hasMultipleConsecutiveSpaces("   ")



import os
from pathlib import Path

from pytest import mark

from ioc_finder import find_iocs, parse_urls

SHORT_TEXT = """abc.py bar.com example.com foo.com swissjabber.de https://example.com/test%20page/foo.com/bingo.php?q=bar.com foo@swissjabber.de me@example.com me@example.com 1.1.1.1/0 imphash 18ddf28a71089acdbab5038f58044c0a authentihash 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4 1.1.1.1 2001:0db8:0000:0000:0000:ff00:0042:8329 aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa 12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F 0000:0000:ff00 2001:0db8:0000 ASN123 CVE-2022-1234 HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows pub-1234567891234567 UA-000000-1 imphash 18ddf28a71089acdbab5038f58044c0a 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy 496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp AA-F2-C9-A6-B3-4F Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1) TLP:RED ~/foo/bar/abc.py enterprise pre_attack pre_attack M1036 M1015 TA0012 T1329"""
LONG_TEXT = Path(os.path.join(os.path.dirname(__file__), "./data/long-article-1.txt")).read_text(encoding="utf8")
TEXTS = (SHORT_TEXT, LONG_TEXT)


def _run(f):
    for text in TEXTS:
        f(text)


def test_benchmarks(benchmark):
    benchmark(_run, find_iocs)


def test_parse_urls(benchmark):
    benchmark(_run, parse_urls)



from pytest import param

EMAIL_DATA = [
    param(
        "a@example.com test@a.com bingo@en.wikipedia.com foo@a.com'.format('a'*63 bar@b.a.com'.format('a'*63, 'a'*63 bad@test-ing.com me@2600.com john.smith(comment)@example.com (comment)john.smith@example.com \"John..Doe\"@example.com' test@[192.168.0.1]",
        {
            "email_addresses": [
                "a@example.com",
                "bad@test-ing.com",
                "bar@b.a.com",
                "bingo@en.wikipedia.com",
                "foo@a.com",
                "john.smith@example.com",
                "me@2600.com",
                "test@[192.168.0.1]",
                "test@a.com",
            ],
            "email_addresses_complete": [
                "a@example.com",
                "test@a.com",
                "bingo@en.wikipedia.com",
                "foo@a.com",
                "bar@b.a.com",
                "bad@test-ing.com",
                "me@2600.com",
                "john.smith(comment)@example.com",
                "(comment)john.smith@example.com",
                '"John..Doe"@example.com',
                "test@[192.168.0.1]",
            ],
            "ipv4s": ["192.168.0.1"],
            "domains": ["a.com", "en.wikipedia.com", "b.a.com", "test-ing.com", "2600.com", "example.com"],
        },
        {},
        id="email_1",
    ),
    param(
        "foo@swissjabber.de bar@jabber.zone bom@jabber.sow.as me@example.com",
        {
            "xmpp_addresses": ["foo@swissjabber.de", "bar@jabber.zone", "bom@jabber.sow.as"],
            "domains": ["swissjabber.de", "jabber.zone", "jabber.sow.as", "example.com"],
            "email_addresses": ["me@example.com"],
            "email_addresses_complete": ["me@example.com"],
        },
        {},
        id="xmpp_1",
    ),
    param(
        "foo@swissjabber.de bar@jabber.zone bom@jabber.sow.as me@example.com",
        {
            "xmpp_addresses": ["foo@swissjabber.de", "bar@jabber.zone", "bom@jabber.sow.as"],
            "email_addresses": ["me@example.com"],
            "email_addresses_complete": ["me@example.com"],
        },
        {"parse_domain_name_from_xmpp_address": False, "parse_domain_from_email_address": False},
        id="xmpp_2",
    ),
]



"""Test the URL parsing against the urls here: https://mathiasbynens.be/demo/url-regex."""

from d8s_lists import iterables_have_same_items

from ioc_finder import find_iocs

# VALID_URLS = [
#     'http://foo.com/blah_blah',
#     'http://foo.com/blah_blah/',
#     'http://foo.com/blah_blah_(wikipedia)',
#     'http://foo.com/blah_blah_(wikipedia)_(again)',
#     'http://www.example.com/wpstyle/?p=364',
#     'https://www.example.com/foo/?bar=baz&inga=42&quux',
#     'http://✪df.ws/123',
#     'http://userid:password@example.com:8080',
#     'http://userid:password@example.com:8080/',
#     'http://userid@example.com',
#     'http://userid@example.com/',
#     'http://userid@example.com:8080',
#     'http://userid@example.com:8080/',
#     'http://userid:password@example.com',
#     'http://userid:password@example.com/',
#     'http://142.42.1.1/',
#     'http://142.42.1.1:8080/',
#     'http://➡.ws/䨹',
#     'http://⌘.ws',
#     'http://⌘.ws/',
#     'http://foo.com/blah_(wikipedia)#cite-1',
#     'http://foo.com/blah_(wikipedia)_blah#cite-1',
#     'http://foo.com/unicode_(✪)_in_parens',
#     'http://foo.com/(something)?after=parens',
#     'http://☺.damowmow.com/',
#     'http://code.google.com/events/#&product=browser',
#     'http://j.mp',
#     'ftp://foo.bar/baz',
#     'http://foo.bar/?q=Test%20URL-encoded%20stuff',
#     'http://مثال.إختبار',
#     'http://例子.测试',
#     'http://उदाहरण.परीक्षा',
#     "http://-.~_!$&'()*+,;=:%40:80%2f::::::@example.com",
#     'http://1337.net',
#     'http://a.b-c.de',
#     'http://223.255.255.254',
# ]


# def test_url_parsing():
#     for url in VALID_URLS:
#         iocs = find_iocs(url)
#         try:
#             assert len(iocs['urls']) == 1
#             assert iocs['urls'][0] == url
#         except AssertionError as e:
#             print('failed on url: {}'.format(url))
#             raise e


# INVALID_URLS = [
#     'http://',
#     'http://.',
#     'http://..',
#     'http://../',
#     'http://?',
#     'http://??',
#     'http://??/',
#     'http://#',
#     'http://##',
#     'http://##/',
#     '//',
#     '//a',
#     '///a',
#     '///',
#     'http:///a',
#     'foo.com',
#     'rdar://1234',
#     'h://test',
#     ':// should fail',
#     'ftps://foo.bar/',
#     'http://-error-.invalid/',
#     'http://a.b--c.de/',
#     'http://-a.b.co',
#     'http://a.b-.co',
#     'http://0.0.0.0',
#     'http://10.1.1.0',
#     'http://10.1.1.255',
#     'http://224.1.1.1',
#     'http://1.1.1.1.1',
#     'http://123.123.123',
#     'http://3628126748',
#     'http://.www.foo.bar/',
#     'http://www.foo.bar./',
#     'http://.www.foo.bar./',
#     'http://10.1.1.1',
# ]


# def test_invalid_urls():
#     for url in INVALID_URLS:
#         iocs = find_iocs(url)
#         assert len(iocs['urls']) == 0


def test_cidr_ranges_not_found_as_urls():
    """See https://github.com/fhightower/ioc-finder/issues/91."""
    result = find_iocs("1.1.1.1/0")
    assert result["urls"] == []

    result = find_iocs("1.1.1.1/0", parse_urls_without_scheme=False)
    assert result["urls"] == []

    result = find_iocs("1.1.1.1/0 foobar.com/test/bingo.php")
    assert result["urls"] == ["foobar.com/test/bingo.php"]


def test_parse_domain_from_url_not_removing_entire_url():
    """See https://github.com/fhightower/ioc-finder/issues/90."""
    # default behaviour
    result = find_iocs("https://foobar.com/test/bingo.com/bar")
    assert iterables_have_same_items(result["domains"], ["foobar.com", "bingo.com"])

    result = find_iocs("https://foobar.com/test/bingo.com/bar", parse_domain_from_url=False)
    assert result["domains"] == ["bingo.com"]

    result = find_iocs("https://foobar.com/test/bingo.com/bar", parse_domain_from_url=False, parse_from_url_path=False)
    assert result["domains"] == []


def test_issue_104__encoded_url_properly_parsed():
    s = "https://asf.goole.com/mail?url=http%3A%2F%2Ffreasdfuewriter.com%2Fcs%2Fimage%2FCommerciaE.jpg&t=1575955624&ymreqid=733bc9eb-e8f-34cb-1cb5-120010019e00&sig=x2Pa2oOYxanG52s4vyCEFg--~Chttp://uniddloos.zddfdd.org/CBA0019_file_00002_pdf.zip"
    result = find_iocs(s)
    assert result["urls"] == [
        "https://asf.goole.com/mail?url=http%3A%2F%2Ffreasdfuewriter.com%2Fcs%2Fimage%2FCommerciaE.jpg&t=1575955624&ymreqid=733bc9eb-e8f-34cb-1cb5-120010019e00&sig=x2Pa2oOYxanG52s4vyCEFg--~Chttp://uniddloos.zddfdd.org/CBA0019_file_00002_pdf.zip"
    ]


def test_url__percent_encoded_path():
    # make sure a percent encoded path is properly removed so that nothing is parsed from it
    s = "https://example.com/test%20page/foo.com/bingo.php?q=bar.com"
    result = find_iocs(s, parse_from_url_path=False)
    assert result["urls"] == ["https://example.com/test%20page/foo.com/bingo.php?q=bar.com"]
    assert iterables_have_same_items(
        result["domains"], ["example.com", "bar.com"]
    )  # the key here is that "foo.com" is not parsed because it is part of the path (which has been removed)
    assert result["file_paths"] == []



from pytest import param

ATTACK_DATA = [
    param(
        """Mitigations: 41
        ID  Name    Description
        M1036   Account Use Policies
        Configure features related to account use like login attempt lockouts, specific login times, etc.

        M1015   Active Directory Configuration
        Configur""",
        {"attack_mitigations": {"enterprise": ["M1036", "M1015"]}},
        {},
        id="attack_data_1",
    ),
    param(
        """ Name    Description
        M1013   Application Developer Guidance
        This mitigation describes any guidance or training given to developers of applications to avoid introducing security weaknesses that an adversary may be able to take advantage of.

        M1005   Application Vetting
        Enterprises can vet applications for exploitable vulnerabilities or unwanted (privacy-invasive or malicious) behaviors. Enterprises can inspect appl""",
        {"attack_mitigations": {"enterprise": ["M1013"], "mobile": ["M1013", "M1005"]}},
        {},
        id="attack_data_2",
    ),
    param(
        """
        ID  Name    Description
        T1329   Acquire and/or use 3rd party infrastructure services
        A wide variety of cloud, virtual private services, hosting, compute, and storage solutions are available. Additionally botnets are available for rent or purchase. Use of these solutions allow an adversary to stage, launch, and execute an attack from infrastructure that does not physically tie back to them and can be rapidly provisioned, modified, and shut down.

        T1307   Acquire and/or use 3rd party infrastructure services
        A wide variety of cloud, virtual private services, hosting, compute, and storage solutions are available. Additionally botnets are available for rent or purchase. Use of these solutions allow an adversary to stage, launch, and execute an attack from infrastructure that does not physically tie back to them and can be rapidly provisioned, modified, and shut down.

        T1308""",
        {"attack_techniques": {"pre_attack": ["T1329", "T1307", "T1308"]}},
        {},
        id="attack_pattern_3",
    ),
    param(
        """
        ID  Name    Description
        TA0012  Priority Definition Planning    Priority definition planning consists of the process of determining the set of Key Intelligence Topics (KIT) or Key Intelligence Questions (KIQ) required for meeting key strategic, operational, or tactical goals. Leadership outlines the priority definition (may be considered a goal) around which the adversary designs target selection and a plan to achieve. An analyst may outline the priority definition when in the course of determining gaps in existing KITs or KIQs.
        TA0013  Priority Definition Direction   Priority definition direction consists of the process of collecting and assigning requirements for meeting Key Intelligence Topics (KIT) or Key Intelligence Questions (KIQ) as determined by leadership.
        TA0014  Targ""",
        {"attack_tactics": {"pre_attack": ["TA0012", "TA0013", "TA0014"]}},
        {},
        id="attack_pattern_4",
    ),
    param("""FOOT1329""", {}, {}, id="attack_pattern_5"),
    param("""T1329FUN""", {}, {}, id="attack_pattern_6"),
    param("""foot1329""", {}, {}, id="attack_pattern_7"),
    param("""t1329""", {"attack_techniques": {"pre_attack": ["T1329"]}}, {}, id="attack_pattern_8"),
    param("T1156", {"attack_techniques": {"enterprise": ["T1156"]}}, {}, id="attack_pattern_9"),
    param("AT0001", {}, {}, id="attack_pattern_10"),
    param("TA0001FUN", {}, {}, id="attack_pattern_11"),
    param("foota0001", {}, {}, id="attack_pattern_12"),
    param("ta0001", {"attack_tactics": {"enterprise": ["TA0001"]}}, {}, id="attack_pattern_13"),
    param(
        "https://attack.mitre.org/tactics/TA0001/",
        {
            "attack_tactics": {"enterprise": ["TA0001"]},
            "urls": ["https://attack.mitre.org/tactics/TA0001/"],
            "domains": ["attack.mitre.org"],
        },
        {},
        id="attack_pattern_14",
    ),
    param("T1546.004", {"attack_techniques": {"enterprise": ["T1546.004"]}}, {}, id="attack_pattern_15"),
    param("T1156.0012", {}, {}, id="attack_pattern_16"),
]



from pytest import param

COIN_DATA = [
    param(
        """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
        P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
        Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq""",
        {
            "bitcoin_addresses": [
                "1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2",
                "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy",
                "bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq",
            ]
        },
        {},
        id="bitcoin_address_1",
    ),
    param(
        """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
        P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
        Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq""",
        {
            "bitcoin_addresses": [
                "1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2",
                "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy",
                "bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq",
            ]
        },
        {},
        id="bitcoin_address_1",
    ),
    param(
        """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
        P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
        Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq""",
        {
            "bitcoin_addresses": [
                "1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2",
                "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy",
                "bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq",
            ]
        },
        {},
        id="bitcoin_address_1",
    ),
    param(
        """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
        P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
        Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq""",
        {
            "bitcoin_addresses": [
                "1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2",
                "3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy",
                "bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq",
            ]
        },
        {},
        id="bitcoin_address_1",
    ),
    param(
        "496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp",
        {
            "monero_addresses": [
                "496aKKdqF1xQSSEzw7wNrkZkDUsCD5cSmNCfVhVgEps52WERBcLDGzdF5UugmFoHMm9xRJdewvK2TFfAJNwEV25rTcVF5Vp"
            ]
        },
        {},
        id="monero_1",
    ),
    param(
        "49Bmp3SfddJRRGNW7GhHyAA2JgcYmZ4EGEix6p3eMNFCd15P2VsK9BHWcZWUNYF3nhf17MoRTRK4j5b7FUMA9zanSn9D3Nk 498s2XeKWYSEhQHGxdMULWdrpaKvSkDsq4855mCuksNL6ez2dk4mMQm8epbr9xvn5LgLPzD5uL9EGeRqWUdEZha1HmZqcyh",
        {
            "monero_addresses": [
                "49Bmp3SfddJRRGNW7GhHyAA2JgcYmZ4EGEix6p3eMNFCd15P2VsK9BHWcZWUNYF3nhf17MoRTRK4j5b7FUMA9zanSn9D3Nk",
                "498s2XeKWYSEhQHGxdMULWdrpaKvSkDsq4855mCuksNL6ez2dk4mMQm8epbr9xvn5LgLPzD5uL9EGeRqWUdEZha1HmZqcyh",
            ]
        },
        {},
        id="monero_2",
    ),
]



from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("requirements.txt") as requirements_file:
    requirements = requirements_file.read().splitlines()

setup(
    name="ioc_finder",
    version="7.3.0",
    description="Python package for finding and parsing indicators of compromise from text.",
    entry_points={"console_scripts": ["ioc-finder=ioc_finder.ioc_finder:cli_find_iocs"]},
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Floyd Hightower",
    author_email="",
    url="https://github.com/fhightower/ioc-finder",
    project_urls={
        "Changelog": "https://github.com/fhightower/ioc-finder/blob/main/CHANGELOG.md",
        "Documentation": "https://hightower.space/ioc-finder/",
        "Source": "https://github.com/fhightower/ioc-finder",
        "Issues": "https://github.com/fhightower/ioc-finder/issues",
        "Sponsor": "https://github.com/sponsors/fhightower",
    },
    packages=find_packages(exclude=("tests", "docs")),
    include_package_data=True,
    install_requires=requirements,
    license="GNU Lesser General Public License v3",
    zip_safe=True,
    keywords="iocs,indicators of compromise,parsing,finding,searching,threat intelligence,malware,threat hunting,observables,domains,domain names,asns,cidr,cidr ranges,ips,ip addresses,urls,email addresses,md5,sha1,sha256,google ads,cve,file paths",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    test_suite="tests",
)
